//
//
//
//
//
//
//
//
//
//
//

`include "or1200_defines.v"
`include "timescale.v"
`include "test_defines.v"

module orsoc_testbench();

parameter dw = `OR1200_OPERAND_WIDTH;
parameter aw = `OR1200_OPERAND_WIDTH;

//system signals
reg 	clk;
reg 	rst;

//Setup dumpfile
`ifdef VCD_DUMP
initial begin
	$dumpfile(`VCD_FILE);
	$dumpvars(`VCD_DEPTH, orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_tmu,
		orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_except,
		orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_ctrl,
		orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_sprs,
		orsoc_testbench.orsoc_top.or1200_top.or1200_pic,
		orsoc_testbench.orsoc_top.intgen0,
		orsoc_testbench.orsoc_top.intgen1,
		orsoc_testbench.orsoc_top.arbiter_dbus,
		orsoc_testbench.orsoc_top.arbiter_ibus);
end
`endif

//Setup values
initial begin
	rst <= !`OR1200_RST_VALUE;
	clk <= 0;
	//init_memory();
end

//clock generator
always begin
	#(10) clk <= !clk;
end

//Reset generator
initial begin
	repeat (10) @(negedge clk);
	rst <= `OR1200_RST_VALUE;
	repeat (10) @(negedge clk);
	rst <= !`OR1200_RST_VALUE;
end

//Monitor specific signals

integer log;
initial begin
	log = $fopen("log.txt", "w");
	$fwrite(log, "Time\t, PC\t, TMU_intr, pic_mask\n");
end
always @(posedge orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_tmu.intr or orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_tmu.pic_mask) begin
	monitor_sign();
end


//Clock cycle counter
//integer cnt = 0;
//always @(posedge clk) begin
//	cnt <= cnt + 1;
//	if (cnt == 40000) begin
//		$finish;
//	end
//end

wire uart_tx;

or1200_monitor
     #(.TEST_NAME_STRING (`TEST_NAME),
       .LOG_DIR("../out"))
   monitor();
   
orsoc_top orsoc_top(
	.clk 		(clk),
	.rst_i 		(rst),
	//JTAG
	.tdo_pad_o	(), 
	.tms_pad_i	(1'b0), 
	.tck_pad_i	(1'b0), 
	.tdi_pad_i	(1'b0),
	//Uart
	.uart_stx_pad_o(uart_tx),
	.uart_srx_pad_i(1'b0)
);

uart_decoder uart_decoder (
	.clk(clk),
	.uart_tx(uart_tx)
);



task init_memory;
	integer i;
begin
	$display("Initializing reg file");
	for (i = 0; i < (1<<5); i = i + 1) begin
		orsoc_top.or1200_top.or1200_cpu.or1200_rf.rf_a.mem[i] = 32'h0000_0000;
		orsoc_top.or1200_top.or1200_cpu.or1200_rf.rf_b.mem[i] = 32'h0000_0000;
	end
end
endtask

task monitor_sign;
begin
	$fwrite(log, "%d,\t%h,\t%b,\t%h\n", $time, orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_ctrl.ex_pc,
				orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_tmu.intr,
				orsoc_testbench.orsoc_top.or1200_top.or1200_cpu.or1200_tmu.pic_mask);
end
endtask




endmodule
