//
//
//
//
//
//
//
//
//
//
//

`include "orsoc_defines.v"
`include "or1200_defines.v"
`include "timescale.v"


module orsoc_top(
	clk, rst_i
	//JTAG signals
	`ifdef GENERIC_TAP
	, tdo_pad_o, tms_pad_i, tck_pad_i, tdi_pad_i
	`endif
	
	`ifdef ONBOARD_SDRAM
	//SDRAM ports
	, sd_adr, sd_ba, sd_cas_n, sd_cke, sd_clk, sd_cs_n
	, sd_dqm, sd_dq, sd_ras_n, sd_we_n
	`endif
	//UART signals
	, uart_srx_pad_i, uart_stx_pad_o
);

parameter dw = `OR1200_OPERAND_WIDTH;
parameter aw = `OR1200_OPERAND_WIDTH;

//system signals
input 							clk;
input 							rst_i;

wire 							rst;
`ifdef POSITIVE_RESET
	assign rst = rst_i;
`elsif NEGATIVE_RESET
	assign rst = ~rst_i;
`endif

//JTAG signals
`ifdef GENERIC_TAP
output							tdo_pad_o;
input							tms_pad_i;
input							tck_pad_i;
input							tdi_pad_i;
`endif

//UART signals
input							uart_srx_pad_i;
output							uart_stx_pad_o;

//common wishbone
wire							wb_clk, wb_rst;

//instruction wishbone
wire							or1200_iwb_ack_i;	// normal termination
wire							or1200_iwb_err_i;	// termination w/ error
wire							or1200_iwb_rty_i;	// termination w/ retry
wire	[dw-1:0]				or1200_iwb_dat_i;	// input data bus
wire							or1200_iwb_cyc_o;	// cycle valid output
wire	[aw-1:0]				or1200_iwb_adr_o;	// address bus outputs
wire							or1200_iwb_stb_o;	// strobe output
wire							or1200_iwb_we_o;	// indicates write transfer
wire	[3:0]					or1200_iwb_sel_o;	// byte select outputs
wire	[dw-1:0]				or1200_iwb_dat_o;	// output data bus
`ifdef OR1200_WB_CAB
wire							or1200_iwb_cab_o;	// indicates consecutive address burst
`endif
`ifdef OR1200_WB_B3
wire 	[2:0]					or1200_iwb_cti_o;	// cycle type identifier
wire 	[1:0]					or1200_iwb_bte_o;	// burst type extension
`endif

// data WISHBONE interface
wire							or1200_dwb_ack_i;	// normal termination
wire							or1200_dwb_err_i;	// termination w/ error
wire							or1200_dwb_rty_i;	// termination w/ retry
wire	[dw-1:0]				or1200_dwb_dat_i;	// input data bus
wire							or1200_dwb_cyc_o;	// cycle valid output
wire	[aw-1:0]				or1200_dwb_adr_o;	// address bus outputs
wire							or1200_dwb_stb_o;	// strobe output
wire							or1200_dwb_we_o;	// indicates write transfer
wire	[3:0]					or1200_dwb_sel_o;	// byte select outputs
wire	[dw-1:0]				or1200_dwb_dat_o;	// output data bus
`ifdef OR1200_WB_CAB
wire							or1200_dwb_cab_o;	// indicates consecutive address burst
`endif
`ifdef OR1200_WB_B3
wire	[2:0]					or1200_dwb_cti_o;	// cycle type identifier
wire	[1:0]					or1200_dwb_bte_o;	// burst type extension
`endif

// debug WISHBONE interface
wire							dbg_wb_ack_i;	// normal termination
wire							dbg_wb_err_i;	// termination w/ error
wire							dbg_wb_rty_i;	// termination w/ retry
wire	[dw-1:0]				dbg_wb_dat_i;	// input data bus
wire							dbg_wb_cyc_o;	// cycle valid output
wire	[aw-1:0]				dbg_wb_adr_o;	// address bus outputs
wire							dbg_wb_stb_o;	// strobe output
wire							dbg_wb_we_o;	// indicates write transfer
wire	[3:0]					dbg_wb_sel_o;	// byte select outputs
wire	[dw-1:0]				dbg_wb_dat_o;	// output data bus
`ifdef OR1200_WB_CAB
wire							dbg_wb_cab_o;	// indicates consecutive address burst
`endif
`ifdef OR1200_WB_B3
wire	[2:0]					dbg_wb_cti_o;	// cycle type identifier
wire	[1:0]					dbg_wb_bte_o;	// burst type extension
`endif

// uart WISHBONE interface
wire	[dw-1:0]				uart_wb_dat_i;	// input data bus
wire	[aw-1:0]				uart_wb_adr_i;	// address bus outputs
wire	[3:0]					uart_wb_sel_i;	// byte select outputs
`ifdef OR1200_WB_B3
wire	[2:0]					uart_wb_cti_i;	// cycle type identifier
wire	[1:0]					uart_wb_bte_i;	// burst type extension
`endif
wire							uart_wb_we_i;	// indicates write transfer
wire							uart_wb_cyc_i;	// cycle valid output
wire							uart_wb_stb_i;	// strobe output
wire	[dw-1:0]				uart_wb_dat_o;	// output data bus
wire							uart_wb_ack_o;	// normal termination
wire							uart_wb_err_o;	// termination w/ error
wire							uart_wb_rty_o;	// termination w/ retry
`ifdef OR1200_WB_CAB
wire							uart_wb_cab_o;	// indicates consecutive address burst
`endif

// ram0 instruction WISHBONE interface
wire	[dw-1:0]				ram0_iwb_dat_i;	// input data bus
wire	[aw-1:0]				ram0_iwb_adr_i;	// address bus outputs
wire	[3:0]					ram0_iwb_sel_i;	// byte select outputs
`ifdef OR1200_WB_B3
wire	[2:0]					ram0_iwb_cti_i;	// cycle type identifier
wire	[1:0]					ram0_iwb_bte_i;	// burst type extension
`endif
wire							ram0_iwb_we_i;	// indicates write transfer
wire							ram0_iwb_cyc_i;	// cycle valid output
wire							ram0_iwb_stb_i;	// strobe output
wire	[dw-1:0]				ram0_iwb_dat_o;	// output data bus
wire							ram0_iwb_ack_o;	// normal termination
wire							ram0_iwb_err_o;	// termination w/ error
wire							ram0_iwb_rty_o;	// termination w/ retry
`ifdef OR1200_WB_CAB
wire							ram0_iwb_cab_o;	// indicates consecutive address burst
`endif

// ram0 data WISHBONE interface
wire	[dw-1:0]				ram0_dwb_dat_i;	// input data bus
wire	[aw-1:0]				ram0_dwb_adr_i;	// address bus outputs
wire	[3:0]					ram0_dwb_sel_i;	// byte select outputs
`ifdef OR1200_WB_B3
wire	[2:0]					ram0_dwb_cti_i;	// cycle type identifier
wire	[1:0]					ram0_dwb_bte_i;	// burst type extension
`endif
wire							ram0_dwb_we_i;	// indicates write transfer
wire							ram0_dwb_cyc_i;	// cycle valid output
wire							ram0_dwb_stb_i;	// strobe output
wire	[dw-1:0]				ram0_dwb_dat_o;	// output data bus
wire							ram0_dwb_ack_o;	// normal termination
wire							ram0_dwb_err_o;	// termination w/ error
wire							ram0_dwb_rty_o;	// termination w/ retry
`ifdef OR1200_WB_CAB
wire							ram0_dwb_cab_o;	// indicates consecutive address burst
`endif

// intgen0 WISHBONE interface
wire	[dw-1:0]				intgen0_wb_dat_i;	// input data bus
wire	[aw-1:0]				intgen0_wb_adr_i;	// address bus outputs
wire	[3:0]					intgen0_wb_sel_i;	// byte select outputs
`ifdef OR1200_WB_B3
wire	[2:0]					intgen0_wb_cti_i;	// cycle type identifier
wire	[1:0]					intgen0_wb_bte_i;	// burst type extension
`endif
wire							intgen0_wb_we_i;	// indicates write transfer
wire							intgen0_wb_cyc_i;	// cycle valid output
wire							intgen0_wb_stb_i;	// strobe output
wire	[dw-1:0]				intgen0_wb_dat_o;	// output data bus
wire							intgen0_wb_ack_o;	// normal termination
wire							intgen0_wb_err_o;	// termination w/ error
wire							intgen0_wb_rty_o;	// termination w/ retry
`ifdef OR1200_WB_CAB
wire							intgen0_wb_cab_o;	// indicates consecutive address burst
`endif

// intgen1 WISHBONE interface
wire	[dw-1:0]				intgen1_wb_dat_i;	// input data bus
wire	[aw-1:0]				intgen1_wb_adr_i;	// address bus outputs
wire	[3:0]					intgen1_wb_sel_i;	// byte select outputs
`ifdef OR1200_WB_B3
wire	[2:0]					intgen1_wb_cti_i;	// cycle type identifier
wire	[1:0]					intgen1_wb_bte_i;	// burst type extension
`endif
wire							intgen1_wb_we_i;	// indicates write transfer
wire							intgen1_wb_cyc_i;	// cycle valid output
wire							intgen1_wb_stb_i;	// strobe output
wire	[dw-1:0]				intgen1_wb_dat_o;	// output data bus
wire							intgen1_wb_ack_o;	// normal termination
wire							intgen1_wb_err_o;	// termination w/ error
wire							intgen1_wb_rty_o;	// termination w/ retry
`ifdef OR1200_WB_CAB
wire							intgen1_wb_cab_o;	// indicates consecutive address burst
`endif

//cpu debug interface
wire							dbg_stall_i;
wire							dbg_ewt_i;
wire	[3:0]					dbg_lss_o;
wire	[1:0]					dbg_is_o;
wire	[10:0]					dbg_wp_o;
wire							dbg_bp_o;
wire							dbg_stb_i;
wire							dbg_we_i;
wire	[aw-1:0]				dbg_adr_i;
wire	[dw-1:0]				dbg_dat_i;
wire	[dw-1:0]				dbg_dat_o;
wire							dbg_ack_o;

//debug jtag interface
wire							dbg_tck_i;
wire							jtag_tdo_o;
wire							dbg_tdo_o;
wire							jtag_shift_dr_i;
wire							jtag_pause_dr_i;
wire							jtag_update_dr_i;
wire							jtag_capture_dr_i;
wire							jtag_debug_select_i;
wire							jtag_trstn_i;
`ifdef GENERIC_TAP
assign	dbg_tck_i = tck_pad_i;
assign	jtag_trstn_i = rst;
`endif


`ifdef OR1200_BIST
//BIST interface
reg								mbist_si_i;
reg		[OR1200_MBIST_CTRL_WIDTH-1:0] mbist_ctrl_i; //OR1200_MBIST_CTRL_WIDTH-1:0
wire							mbist_so_o;
`endif


//power management interface
wire							pm_cpustall_i;
wire	[3:0]					pm_clksd_o;
wire							pm_dc_gate_o;
wire							pm_ic_gate_o;
wire							pm_dmmu_gate_o;
wire							pm_immu_gate_o;
wire							pm_tt_gate_o;
wire							pm_cpu_gate_o;
wire							pm_wakeup_o;
wire							pm_lvolt_o;

wire          					sig_tick_o;
wire 	[`OR1200_PIC_INTS-1:0]	pic_ints_i;

assign wb_rst = rst;
assign wb_clk = clk;

//instruction wishbone bus arbiter
arbiter_ibus #(
	.wb_addr_match_width(4),
	.slave0_adr(4'hf),
	.slave1_adr(4'h0)
) arbiter_ibus (
	.wb_clk			(wb_clk),
	.wb_rst			(wb_rst),	
	// instruction bus in
	// Wishbone Master interface
	.wbm_adr_o		(or1200_iwb_adr_o),
	.wbm_dat_o		(or1200_iwb_dat_o),
	.wbm_sel_o		(or1200_iwb_sel_o),
	.wbm_we_o		(or1200_iwb_we_o),
	.wbm_cyc_o		(or1200_iwb_cyc_o),
	.wbm_stb_o		(or1200_iwb_stb_o),
	.wbm_cti_o		(or1200_iwb_cti_o),
	.wbm_bte_o		(or1200_iwb_bte_o),
	.wbm_dat_i		(or1200_iwb_dat_i),
	.wbm_ack_i		(or1200_iwb_ack_i),
	.wbm_err_i		(or1200_iwb_err_i),
	.wbm_rty_i		(or1200_iwb_rty_i),

	// Slave one
	// Wishbone Slave interface
	.wbs0_adr_i		(),
	.wbs0_dat_i		(),
	.wbs0_sel_i		(),
	.wbs0_we_i		(),
	.wbs0_cyc_i		(),
	.wbs0_stb_i		(),
	.wbs0_cti_i		(),
	.wbs0_bte_i		(),
	.wbs0_dat_o		(32'h0000_0000),
	.wbs0_ack_o		(1'b0),
	.wbs0_err_o		(1'b1),
	.wbs0_rty_o		(1'b0),

	// Slave two
	// Wishbone Slave interface
	.wbs1_adr_i		(ram0_iwb_adr_i),
	.wbs1_dat_i		(ram0_iwb_dat_i),
	.wbs1_sel_i		(ram0_iwb_sel_i),
	.wbs1_we_i		(ram0_iwb_we_i),
	.wbs1_cyc_i		(ram0_iwb_cyc_i),
	.wbs1_stb_i		(ram0_iwb_stb_i),
	.wbs1_cti_i		(ram0_iwb_cti_i),
	.wbs1_bte_i		(ram0_iwb_bte_i),
	.wbs1_dat_o		(ram0_iwb_dat_o),
	.wbs1_ack_o		(ram0_iwb_ack_o),
	.wbs1_err_o		(ram0_iwb_err_o),
	.wbs1_rty_o		(ram0_iwb_rty_o)
);

arbiter_dbus #(
	.wb_num_slaves(`ARB_NUM_SLAVES),
	.wb_addr_match_width(`ARB_ADDR_WIDTH),
	.slave0_adr(`SLAVE0_ADDR),
	.slave1_adr(`SLAVE1_ADDR),
	.slave2_adr(`SLAVE2_ADDR),
	.slave3_adr(`SLAVE3_ADDR)
) arbiter_dbus (
	.wb_clk			(wb_clk),
	.wb_rst			(wb_rst),
	// or1200 data master
	// Wishbone Master interface
	.wbm0_adr_o		(or1200_dwb_adr_o),
	.wbm0_dat_o		(or1200_dwb_dat_o),
	.wbm0_sel_o		(or1200_dwb_sel_o),
	.wbm0_we_o		(or1200_dwb_we_o),
	.wbm0_cyc_o		(or1200_dwb_cyc_o),
	.wbm0_stb_o		(or1200_dwb_stb_o),
	.wbm0_cti_o		(or1200_dwb_cti_o),
	.wbm0_bte_o		(or1200_dwb_bte_o),
	.wbm0_dat_i		(or1200_dwb_dat_i),
	.wbm0_ack_i		(or1200_dwb_ack_i),
	.wbm0_err_i		(or1200_dwb_err_i),
	.wbm0_rty_i		(or1200_dwb_rty_i),

	// or1200 debug master
	// Wishbone Master interface
	.wbm1_adr_o		(dbg_wb_adr_o),
	.wbm1_dat_o		(dbg_wb_dat_o),
	.wbm1_sel_o		(dbg_wb_sel_o),
	.wbm1_we_o		(dbg_wb_we_o),
	.wbm1_cyc_o		(dbg_wb_cyc_o),
	.wbm1_stb_o		(dbg_wb_stb_o),
	.wbm1_cti_o		(dbg_wb_cti_o),
	.wbm1_bte_o		(dbg_wb_bte_o),
	.wbm1_dat_i		(dbg_wb_dat_i),
	.wbm1_ack_i		(dbg_wb_ack_i),
	.wbm1_err_i		(dbg_wb_err_i),
	.wbm1_rty_i		(dbg_wb_rty_i),

	// Slave one, ram
	// Wishbone Slave interface
	.wbs0_adr_i		(ram0_dwb_adr_i),
	.wbs0_dat_i		(ram0_dwb_dat_i),
	.wbs0_sel_i		(ram0_dwb_sel_i),		    
	.wbs0_we_i		(ram0_dwb_we_i),
	.wbs0_cyc_i		(ram0_dwb_cyc_i),
	.wbs0_stb_i		(ram0_dwb_stb_i),
	.wbs0_cti_i		(ram0_dwb_cti_i),
	.wbs0_bte_i		(ram0_dwb_bte_i),
	.wbs0_dat_o		(ram0_dwb_dat_o),
	.wbs0_ack_o		(ram0_dwb_ack_o),
	.wbs0_err_o		(ram0_dwb_err_o),
	.wbs0_rty_o		(ram0_dwb_rty_o),

	// Slave two, uart
	// Wishbone Slave interface
	.wbs1_adr_i		(uart_wb_adr_i),
	.wbs1_dat_i		(uart_wb_dat_i),
	.wbs1_sel_i		(uart_wb_sel_i),
	.wbs1_we_i		(uart_wb_we_i),
	.wbs1_cyc_i		(uart_wb_cyc_i),
	.wbs1_stb_i		(uart_wb_stb_i),
	.wbs1_cti_i		(uart_wb_cti_i),
	.wbs1_bte_i		(uart_wb_bte_i),
	.wbs1_dat_o		(uart_wb_dat_o),
	.wbs1_ack_o		(uart_wb_ack_o),
	.wbs1_err_o		(uart_wb_err_o),
	.wbs1_rty_o		(uart_wb_rty_o),
	
	// Slave three, intgen0
	// Wishbone Slave interface
	.wbs2_adr_i		(intgen0_wb_adr_i),
	.wbs2_dat_i		(intgen0_wb_dat_i),
	.wbs2_sel_i		(intgen0_wb_sel_i),
	.wbs2_we_i		(intgen0_wb_we_i),
	.wbs2_cyc_i		(intgen0_wb_cyc_i),
	.wbs2_stb_i		(intgen0_wb_stb_i),
	.wbs2_cti_i		(intgen0_wb_cti_i),
	.wbs2_bte_i		(intgen0_wb_bte_i),
	.wbs2_dat_o		(intgen0_wb_dat_o),
	.wbs2_ack_o		(intgen0_wb_ack_o),
	.wbs2_err_o		(intgen0_wb_err_o),
	.wbs2_rty_o		(intgen0_wb_rty_o),

	// Slave four, intgen1
	// Wishbone Slave interface
	.wbs3_adr_i		(intgen1_wb_adr_i),
	.wbs3_dat_i		(intgen1_wb_dat_i),
	.wbs3_sel_i		(intgen1_wb_sel_i),
	.wbs3_we_i		(intgen1_wb_we_i),
	.wbs3_cyc_i		(intgen1_wb_cyc_i),
	.wbs3_stb_i		(intgen1_wb_stb_i),
	.wbs3_cti_i		(intgen1_wb_cti_i),
	.wbs3_bte_i		(intgen1_wb_bte_i),
	.wbs3_dat_o		(intgen1_wb_dat_o),
	.wbs3_ack_o		(intgen1_wb_ack_o),
	.wbs3_err_o		(intgen1_wb_err_o),
	.wbs3_rty_o		(intgen1_wb_rty_o)

);

`ifdef FPGA_TAP
`ifdef ALTERA_FPGA
	altera_virtual_jtag jtag_top (
		.tck_o              (dbg_tck_i),
		.debug_tdo_i        (dbg_tdo_o),
		.tdi_o              (jtag_tdo_o),
		.test_logic_reset_o (jtag_trstn_i),
		.run_test_idle_o    (),
		
		.shift_dr_o         (jtag_shift_dr_i),
		.capture_dr_o       (jtag_capture_dr_i),
		.pause_dr_o         (jtag_pause_dr_i),
		.update_dr_o        (jtag_update_dr_i),
		.debug_select_o     (jtag_debug_select_i) 
	);
`endif
`else
tap_top jtag_tap (
	// JTAG pads
	.tms_pad_i				(tms_pad_i), 
	.tck_pad_i				(tck_pad_i), 
	.trstn_pad_i			(jtag_trstn_i), 
	.tdi_pad_i				(tdi_pad_i), 
	.tdo_pad_o				(tdo_pad_o), 
	.tdo_padoe_o			(),

	// TAP states
	.test_logic_reset_o		(),
	.run_test_idle_o		(),
	.shift_dr_o				(jtag_shift_dr_i),
	.pause_dr_o				(jtag_pause_dr_i), 
	.update_dr_o			(jtag_update_dr_i),
	.capture_dr_o			(jtag_capture_dr_i),

	// Select signals for boundary scan or mbist
	.extest_select_o		(), 
	.sample_preload_select_o(),
	.mbist_select_o			(),
	.debug_select_o			(jtag_debug_select_i),

	// TDO signal that is connected to TDI of sub-modules.
	.tdo_o					(jtag_tdo_o), 

	// TDI signals from sub-modules
	.debug_tdo_i			(1'b0),    // from debug module
	.bs_chain_tdo_i			(1'b0), // from Boundary Scan Chain
	.mbist_tdo_i			(dbg_tdo_o)     // from Mbist Chain
);
`endif

adbg_top adbg_if (
	// JTAG signals
	.tck_i(				dbg_tck_i),
	.tdi_i(				jtag_tdo_o),
	.tdo_o(				dbg_tdo_o),
	.rst_i(				jtag_trstn_i),

	// TAP states
	.shift_dr_i(		jtag_shift_dr_i),
	.pause_dr_i(		jtag_pause_dr_i),
	.update_dr_i(		jtag_update_dr_i),
	.capture_dr_i(		jtag_capture_dr_i),

	// Instructions
	.debug_select_i(	jtag_debug_select_i),

	// WISHBONE common signals
	.wb_clk_i(	wb_clk			),
	.wb_rst_i(	wb_rst			),
	// WISHBONE master interface
	.wb_adr_o(	dbg_wb_adr_o	),
	.wb_dat_o(	dbg_wb_dat_o	),
	.wb_dat_i(	dbg_wb_dat_i	),
	.wb_cyc_o(	dbg_wb_cyc_o	),
	.wb_stb_o(	dbg_wb_stb_o	),
	.wb_sel_o(	dbg_wb_sel_o	),
	.wb_we_o(	dbg_wb_we_o		),
	.wb_ack_i(	dbg_wb_ack_i	),
	.wb_cab_o(	dbg_wb_cab_o	),
	.wb_err_i(	dbg_wb_err_i	),
	.wb_cti_o(	dbg_wb_cti_o	),
	.wb_bte_o(	dbg_wb_bte_o	),

	// CPU signals
	.cpu0_clk_i(	clk			), 
	.cpu0_addr_o(	dbg_adr_i	), 
	.cpu0_data_i(	dbg_dat_o	), 
	.cpu0_data_o(	dbg_dat_i	),
	.cpu0_bp_i(		dbg_bp_o	),
	.cpu0_stall_o(	dbg_stall_i	),
	.cpu0_stb_o(	dbg_stb_i	),
	.cpu0_we_o(		dbg_we_i	),
	.cpu0_ack_i(	dbg_ack_o	),
	.cpu0_rst_o(	dbg_rst_i	)
);


assign 	uart_wb_err_o = 1'b0;
assign 	uart_wb_rty_o = 1'b0;
wire	uart_interrupt;

uart_top uart16550 (
	.wb_clk_i		(wb_clk), 
	.wb_rst_i		(wb_rst),
	// Wishbone signals
	.wb_adr_i		(uart_wb_adr_i[4:0]),
	.wb_dat_i		(uart_wb_dat_i),
	.wb_dat_o		(uart_wb_dat_o),
	.wb_we_i		(uart_wb_we_i),
	.wb_stb_i		(uart_wb_stb_i),
	.wb_cyc_i		(uart_wb_cyc_i),
	.wb_ack_o		(uart_wb_ack_o),
	.wb_sel_i		(uart_wb_sel_i),
	// interrupt request
	.int_o			(uart_interrupt), 
	// UART	signals
	// serial input/output
	.stx_pad_o		(uart_stx_pad_o),
	.srx_pad_i		(uart_srx_pad_i),

	// modem signals
	.rts_pad_o		(),
	.cts_pad_i		(1'b0),
	.dtr_pad_o		(),
	.dsr_pad_i		(1'b0),
	.ri_pad_i		(1'b0),
	.dcd_pad_i		(1'b0)

);



`ifdef ONBOARD_SDRAM

wire	[15:0]	sd_dq_i;
wire	[15:0]	sd_dq_o;
wire				sq_dq_oe;

assign sd_dq_i = sd_dq;
assign sd_dq = sd_dq_oe ? sd_dq_o : 16'bz;
assign sd_clk = wb_clk;

 versatile_mem_ctrl versatile_mem_ctrl (
	// External SDRAM interface
	.ba_pad_o			(sd_ba),
	.a_pad_o				(sd_adr),
	.cs_n_pad_o			(sd_cs_n),
	.ras_pad_o			(sd_ras_n),
	.cas_pad_o			(sd_cas_n),
	.we_pad_o			(sd_we_n),
	.dq_i					(sd_dq_i),      
	.dq_o					(sd_dq_o),
	.dqm_pad_o			(sd_dqm),
	.dq_oe				(sd_dq_oe),
	.cke_pad_o			(sd_cke),
	.sdram_clk			(wb_clk),           
	.sdram_rst        (wb_rst),
	
	//Wishbone slave interface 0
	.wb_dat_i_0			({{ram0_iwb_dat_i,ram0_iwb_sel_i},{ram0_dwb_dat_i,ram0_dwb_sel_i}}),
	.wb_adr_i_0			({{ram0_iwb_adr_i[31:2], ram0_iwb_we_i, ram0_iwb_bte_i, ram0_iwb_cti_i},
							{ram0_dwb_adr_i[31:2],ram0_dwb_we_i, ram0_dwb_bte_i, ram0_dwb_cti_i}}),
	.wb_cyc_i_0			({ram0_iwb_cyc_i, ram0_dwb_cyc_i}),
	.wb_stb_i_0			({ram0_iwb_stb_i, ram0_dwb_stb_i}),
	.wb_dat_o_0			({ram0_iwb_dat_o, ram0_dwb_dat_o}),
	.wb_ack_o_0			({ram0_iwb_ack_o, ram0_dwb_ack_o}),
	// Wishbone slave interface 1
	.wb_dat_i_1			(2'd0),
	.wb_adr_i_1			(2'd0),
	.wb_cyc_i_1			(2'd0),
	.wb_stb_i_1			(2'd0),
	.wb_dat_o_1			(),
	.wb_ack_o_1			(),

	// Wishbone slave interface 2
	.wb_dat_i_2			(2'd0),
	.wb_adr_i_2			(2'd0),
	.wb_cyc_i_2			(2'd0),
	.wb_stb_i_2			(2'd0),
	.wb_dat_o_2			(),
	.wb_ack_o_2			(),
	
	// Wishbone slave interface 3
	.wb_dat_i_3			(2'd0),
	.wb_adr_i_3			(2'd0),
	.wb_cyc_i_3			(2'd0),
	.wb_stb_i_3			(2'd0),
	.wb_dat_o_3			(),
	.wb_ack_o_3			(),

	.wb_clk				(wb_clk),
	.wb_rst				(wb_rst)
);
	defparam versatile_mem_ctrl.nr_of_wb_ports_clk0  = 2;
	assign ram0_iwb_rty_o = 0;
	assign ram0_iwb_err_o = 0;
	assign ram0_dwb_rty_o = 0;
	assign ram0_dwb_err_o = 0;
`else

ram_wb #(
	.mem_size_bytes(`MEM_SIZE_BYTES),
	.mem_adr_width(`MEM_ADDR_WIDTH)
) ram0 (
      // Wishbone slave interface 0, instruction
      .wbm0_dat_i			(ram0_iwb_dat_i),
      .wbm0_adr_i			(ram0_iwb_adr_i),
      .wbm0_sel_i			(ram0_iwb_sel_i),
      .wbm0_cti_i			(ram0_iwb_cti_i),
      .wbm0_bte_i			(ram0_iwb_bte_i),
      .wbm0_we_i			(ram0_iwb_we_i),
      .wbm0_cyc_i			(ram0_iwb_cyc_i),
      .wbm0_stb_i			(ram0_iwb_stb_i),
      .wbm0_dat_o			(ram0_iwb_dat_o),
      .wbm0_ack_o			(ram0_iwb_ack_o),
      .wbm0_err_o           (ram0_iwb_err_o),
      .wbm0_rty_o           (ram0_iwb_rty_o),
      // Wishbone slave interface 1, data
      .wbm1_dat_i			(ram0_dwb_dat_i),
      .wbm1_adr_i			(ram0_dwb_adr_i),
      .wbm1_sel_i			(ram0_dwb_sel_i),
      .wbm1_cti_i			(ram0_dwb_cti_i),
      .wbm1_bte_i			(ram0_dwb_bte_i),
      .wbm1_we_i			(ram0_dwb_we_i),
      .wbm1_cyc_i			(ram0_dwb_cyc_i),
      .wbm1_stb_i			(ram0_dwb_stb_i),
      .wbm1_dat_o			(ram0_dwb_dat_o),
      .wbm1_ack_o			(ram0_dwb_ack_o),
      .wbm1_err_o           (ram0_dwb_err_o),
      .wbm1_rty_o           (ram0_dwb_rty_o),
      // Wishbone slave interface 2
      .wbm2_dat_i			(32'd0),
      .wbm2_adr_i			(32'd0),
      .wbm2_sel_i			(4'd0),
      .wbm2_cti_i			(3'd0),
      .wbm2_bte_i			(2'd0),
      .wbm2_we_i			(1'd0),
      .wbm2_cyc_i			(1'd0),
      .wbm2_stb_i			(1'd0),
      .wbm2_dat_o			(),
      .wbm2_ack_o			(),
      .wbm2_err_o           (),
      .wbm2_rty_o           (),
      // Clock, reset
      .wb_clk_i				(wb_clk),
      .wb_rst_i				(wb_rst)
);

`endif
assign pm_cpustall_i = 1'b0;

or1200_top or1200_top (	
	//system signals
	.clk_i			(clk),
	.rst_i			(rst),
	.pic_ints_i		(pic_ints_i),	//ppic_ints-1:0
	.clmode_i		(2'b00),			//1:0
	
	//Instruction WISHBONE interface
	
	.iwb_clk_i		(wb_clk),
	.iwb_rst_i		(wb_rst),
	.iwb_ack_i		(or1200_iwb_ack_i),
	.iwb_err_i		(or1200_iwb_err_i),
	.iwb_rty_i		(or1200_iwb_rty_i),
	.iwb_dat_i		(or1200_iwb_dat_i), 	//dw-1:0
	.iwb_cyc_o		(or1200_iwb_cyc_o),
	.iwb_adr_o		(or1200_iwb_adr_o), 	//aw-1:0
	.iwb_stb_o		(or1200_iwb_stb_o),
	.iwb_we_o		(or1200_iwb_we_o),
	.iwb_sel_o		(or1200_iwb_sel_o), 	//3:0
	.iwb_dat_o		(or1200_iwb_dat_o), 	//dw-1:0

`ifdef OR1200_WB_CAB
	.iwb_cap_o		(or1200_iwb_cap_o),
`endif
`ifdef OR1200_WB_B3
	.iwb_cti_o		(or1200_iwb_cti_o), 	//2:0
	.iwb_bte_o		(or1200_iwb_bte_o), 	//1:0
`endif

	//Data WISHBONE interface
	.dwb_clk_i		(wb_clk),
	.dwb_rst_i		(wb_rst),
	.dwb_ack_i		(or1200_dwb_ack_i),
	.dwb_err_i		(or1200_dwb_err_i),
	.dwb_rty_i		(or1200_dwb_rty_i),
	.dwb_dat_i		(or1200_dwb_dat_i), 	//dw-1:0
	.dwb_cyc_o		(or1200_dwb_cyc_o),
	.dwb_adr_o		(or1200_dwb_adr_o), 	//aw-1:0
	.dwb_stb_o		(or1200_dwb_stb_o),
	.dwb_we_o		(or1200_dwb_we_o),
	.dwb_sel_o		(or1200_dwb_sel_o), 	//3:0
	.dwb_dat_o		(or1200_dwb_dat_o), 	//dw-1:0
`ifdef OR1200_WB_CAB
	.dwb_cap_o		(or1200_dwb_cap_o),
`endif
`ifdef OR1200_WB_B3
	.dwb_cti_o		(or1200_dwb_cti_o), 	//2:0
	.dwb_bte_o		(or1200_dwb_bte_o), 	//1:0
`endif
	
	//Debug interface
	.dbg_stall_i	(dbg_stall_i),
	.dbg_ewt_i		(1'b0),
	.dbg_lss_o		(dbg_lss_o), 	//3:0
	.dbg_is_o		(dbg_is_o), 	//1:0
	.dbg_wp_o		(dbg_wp_o),	//10:0
	.dbg_bp_o		(dbg_bp_o),
	.dbg_stb_i		(dbg_stb_i),
	.dbg_we_i		(dbg_we_i),
	.dbg_adr_i		(dbg_adr_i),	//aw-1:0
	.dbg_dat_i		(dbg_dat_i),	//dw-1:0
	.dbg_dat_o		(dbg_dat_o),	//dw-1:0
	.dbg_ack_o		(dbg_ack_o),
	
`ifdef OR1200_BIST
	.mbist_si_i		(mbist_si_i	),
	.mbist_ctrl_i	(mbist_ctrl_i), //OR1200_MBIST_CTRL_WIDTH-1:0
	.mbist_so_o		(mbist_so_o),
`endif

	//Power management
	.pm_cpustall_i	(pm_cpustall_i),
	.pm_clksd_o		(pm_clksd_o	),		//3:0
	.pm_dc_gate_o	(pm_dc_gate_o),
	.pm_ic_gate_o	(pm_ic_gate_o),
	.pm_dmmu_gate_o	(pm_dmmu_gate_o),
	.pm_immu_gate_o	(pm_immu_gate_o),
	.pm_tt_gate_o	(pm_tt_gate_o),
	.pm_cpu_gate_o	(pm_cpu_gate_o),
	.pm_wakeup_o	(pm_wakeup_o),
	.pm_lvolt_o		(pm_lvolt_o),
	
	
	.sig_tick (sig_tick_o)
);


wire        intgen0_irq;
intgen intgen0 (
	.clk_i			(wb_clk),
	.rst_i			(wb_rst),
	.wb_adr_i		(intgen0_wb_adr_i[2]),
	.wb_cyc_i		(intgen0_wb_cyc_i),
	.wb_stb_i		(intgen0_wb_stb_i),
	.wb_dat_i		(intgen0_wb_dat_i),
	.wb_we_i			(intgen0_wb_we_i),
	.wb_ack_o		(intgen0_wb_ack_o),
	.wb_dat_o		(intgen0_wb_dat_o),

	.irq_o			(intgen0_irq)
);

assign intgen0_wb_err_o = 0;
assign intgen0_wb_rty_o = 0;
assign intgen0_wb_adr_i[31:3] = 0;
assign intgen0_wb_adr_i[1:0] = 0;
//assign intgen0_wb_dat_i[31:16] = 0;
//assign intgen0_wb_dat_o[31:16] = 0;

wire        intgen1_irq;
intgen intgen1 (
	.clk_i			(wb_clk),
	.rst_i			(wb_rst),
	.wb_adr_i		(intgen1_wb_adr_i[2]),
	.wb_cyc_i		(intgen1_wb_cyc_i),
	.wb_stb_i		(intgen1_wb_stb_i),
	.wb_dat_i		(intgen1_wb_dat_i),
	.wb_we_i			(intgen1_wb_we_i),
	.wb_ack_o		(intgen1_wb_ack_o),
	.wb_dat_o		(intgen1_wb_dat_o),

	.irq_o			(intgen1_irq)
);

assign intgen1_wb_err_o = 0;
assign intgen1_wb_rty_o = 0;
assign intgen1_wb_adr_i[31:3] = 0;
assign intgen1_wb_adr_i[1:0] = 0;
//assign intgen1_wb_dat_i[31:16] = 0;
//assign intgen1_wb_dat_o[31:16] = 0;

//interrupt assignments
assign pic_ints_i[0] = 0;//Non maskable
assign pic_ints_i[1] = 0;//Non maskable
assign pic_ints_i[2] = uart_interrupt;
assign pic_ints_i[3] = intgen0_irq;
assign pic_ints_i[4] = intgen1_irq;
assign pic_ints_i[5] = 0;
assign pic_ints_i[6] = 0;
assign pic_ints_i[7] = 0;
assign pic_ints_i[8] = 0;
assign pic_ints_i[9] = 0;
assign pic_ints_i[10] = 0;
assign pic_ints_i[11] = 0;
assign pic_ints_i[12] = 0;
assign pic_ints_i[13] = 0;
assign pic_ints_i[14] = 0;
assign pic_ints_i[15] = 0;
assign pic_ints_i[16] = 0;
assign pic_ints_i[17] = 0;
assign pic_ints_i[18] = 0;
assign pic_ints_i[19] = 0;



endmodule
