/*
	Test the interrupt generators with masking


*/

#include "spr-defs.h"
#include "board.h"
#include "or1200-defines.h"

/* =================================================== [ exceptions ] === */
	.section .vectors, "ax"


/* ---[ 0x100: RESET exception ]----------------------------------------- */
    .org 0x100 	
	l.movhi r0, 0
	/* Clear status register */
	l.ori r1, r0, SPR_SR_SM
	l.mtspr r0, r1, SPR_SR
	/* Clear timer  */
	l.mtspr r0, r0, SPR_TTMR

	/* Jump to program initialisation code */
	.global _start
	l.movhi r4, hi(_start)
	l.ori r4, r4, lo(_start)
	l.jr    r4
	l.nop
/* ---[ 0x800: external interrupt exception ]-------------------------- */
    .org 0x800
    .global _exint_exception
_exint_exception:
	l.movhi r0, 0
	l.addi r12, r12, 0x1
	l.nop
	l.mfspr r3, r0, SPR_PICSR
	//Mask out intgen 0 as source
	l.andi r4, r3, 0x8
	l.sfeq r4, r0
	l.bf 0x9//Branc to next source
	//Clear intgen 0
	l.movhi	r5, hi(0xa0000004)
	l.ori r5, r5, lo(0xa0000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Reset timer 0
	l.movhi	r3, hi(0xa0000000)
	l.ori r3, r3, lo(0xa0000000)
	l.addi r4, r0, 0x00ff
	l.sw 0(r3), r4
	
	//Mask out intgen 1 as source
	l.andi r4, r3, 0x10
	l.sfeq r4, r0
	l.bf 0x9 //Branch to return
	//Clear intgen 1
	l.movhi	r5, hi(0xa1000004)
	l.ori r5, r5, lo(0xa1000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Integen 1
	l.movhi	r3, hi(0xa1000000)
	l.ori r3, r3, lo(0xa1000000)
	l.addi r4, r0, 0x00af
	l.sw 0(r3), r4
	
	l.addi r3, r0, 0x00
	l.mtspr r0, r3, SPR_PICSR
	l.rfe
	l.nop
	
		
/* ---[ 0xf00: TMU exception ]----------------------------------------- */
        .org 0xf00
    .global _tmu_exception
_tmu_exception: 	
	l.movhi r0, 0
	l.nop
	l.jr r10//Jump to next test
	l.nop

/* =================================================== [ text ] === */
	.section .text

/* =================================================== [ start ] === */	

	.global _start
_start:	
	l.jal _cache_init //Found in cache.S part of  or1200 driver
	l.nop
	
	// Kick off test
	l.jal   _main
	l.nop
	
/* =================================================== [ main ] === */	

_main:
	l.add r3, r0, r1
	l.nop 0x02
	l.addi r12, r0, 0x0
	l.addi r13, r0, 0x000000f0
	l.nop

_enable_interrupts:
	l.mfspr r3, r0, SPR_SR
	l.ori r3, r3, SPR_SR_IEE
	l.mtspr r0, r3, SPR_SR
	
_setup_replenishment:
	//Set the replnishment registers
	l.addi r4, r0, 0x0
	l.movhi r5, hi(0x00005000)
	l.ori r5, r5, lo(0x00005000)
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_REPCMP_H
	l.mtspr r0, r5, SPR_TMU_REPCMP_L

_setup_interrupt_limits:
	l.addi r5, r0, 0x10
	//Write to compare register
	l.mtspr r0, r5, SPR_TMU_INTRCMP_BASE+3
	l.mtspr r0, r5, SPR_TMU_INTRCMP_BASE+4

_setup_pic:
	l.addi r4, r0, 0x1f
	l.mtspr r0, r4, SPR_PICMR
	
_setup_intgens:
	//Intgen 0
	l.movhi	r3, hi(0xa0000000)
	l.ori r3, r3, lo(0xa0000000)
	l.addi r4, r0, 0x00ff
	l.sw 0(r3), r4
	//Integen 1
	l.movhi	r3, hi(0xa1000000)
	l.ori r3, r3, lo(0xa1000000)
	l.addi r4, r0, 0x00af
	l.sw 0(r3), r4
	
_wait:
	l.sfgeu r12, r13
	l.bf 0x3
//	l.mfspr r3, r0, SPR_SR
//	l.ori r3, r3, SPR_SR_IEE
//	l.mtspr r0, r3, SPR_SR
	l.j _wait
	l.nop
	l.j _stop_test
	l.nop

_reset_integen0:
	//Clear intgen 0
	l.movhi	r5, hi(0xa0000004)
	l.ori r5, r5, lo(0xa0000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Reset timer 0
	l.movhi	r3, hi(0xa0000000)
	l.ori r3, r3, lo(0xa0000000)
	l.addi r4, r0, 0x00ff
	l.sw 0(r3), r4
	l.j _wait
	l.nop
	
_reset_intgen1:
	//Clear intgen 1
	l.movhi	r5, hi(0xa1000004)
	l.ori r5, r5, lo(0xa1000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Integen 1
	l.movhi	r3, hi(0xa1000000)
	l.ori r3, r3, lo(0xa1000000)
	l.addi r4, r0, 0x00af
	l.sw 0(r3), r4
	l.j _wait
	l.nop
	
_stop_test:
	l.movhi r3, 0x8000
	l.ori r3, r3, 0x000d
	l.nop 0x2
	l.movhi r3, 0x0
	l.nop 0x1
	l.nop
	


	
		
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
