/*
	Test the interrupt generators with the TMU running
	in normal operation
	r10: address of next test
	r12: interrupt exception count
	r13: interrupt exception limit
	r14: tmu exception counter
	r15: tmu exception limit


*/

#include "spr-defs.h"
#include "board.h"
#include "or1200-defines.h"

/* =================================================== [ exceptions ] === */
	.section .vectors, "ax"


/* ---[ 0x100: RESET exception ]----------------------------------------- */
    .org 0x100 	
	l.movhi r0, 0
	l.movhi r1, 0
	l.movhi r2, 0
	l.movhi r3, 0
	l.movhi r4, 0
	l.movhi r5, 0
	l.movhi r6, 0
	l.movhi r7, 0
	l.movhi r8, 0
	l.movhi r9, 0
	l.movhi r10, 0
	l.movhi r11, 0
	l.movhi r12, 0
	l.movhi r13, 0
	l.movhi r14, 0
	l.movhi r15, 0
	l.movhi r16, 0
	l.movhi r17, 0
	l.movhi r18, 0
	l.movhi r19, 0
	l.movhi r20, 0
	l.movhi r21, 0
	l.movhi r22, 0
	l.movhi r23, 0
	l.movhi r24, 0
	l.movhi r25, 0
	l.movhi r26, 0
	l.movhi r27, 0
	l.movhi r28, 0
	l.movhi r29, 0
	l.movhi r30, 0
	l.movhi r31, 0
	/* Clear status register */
	l.ori r1, r0, SPR_SR_SM
	l.mtspr r0, r1, SPR_SR
	/* Clear timer  */
	l.mtspr r0, r0, SPR_TTMR
	

	/* Jump to program initialisation code */
	.global _start
	l.movhi r4, hi(_start)
	l.ori r4, r4, lo(_start)
	l.jr    r4
	l.nop
/* ---[ 0x800: external interrupt exception ]-------------------------- */
    .org 0x800
    .global _exint_exception
_exint_exception:
	l.movhi r0, 0
	l.addi r12, r12, 0x1
	l.nop
	l.mfspr r3, r0, SPR_PICSR
	//Mask out intgen 0 as source
	l.andi r4, r3, 0x8
	l.sfeq r4, r0
	l.bf 0x9//Branc to next source
	//Clear intgen 0
	l.movhi	r5, hi(0xa0000004)
	l.ori r5, r5, lo(0xa0000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Reset timer 0
	l.movhi	r3, hi(0xa0000000)
	l.ori r3, r3, lo(0xa0000000)
	l.addi r4, r0, 0x00ff
	l.sw 0(r3), r4
	
	//Mask out intgen 1 as source
	l.andi r4, r3, 0x10
	l.sfeq r4, r0
	l.bf 0x9 //Branch to return
	//Clear intgen 1
	l.movhi	r5, hi(0xa1000004)
	l.ori r5, r5, lo(0xa1000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Integen 1
	l.movhi	r3, hi(0xa1000000)
	l.ori r3, r3, lo(0xa1000000)
	l.addi r4, r0, 0x00af
	l.sw 0(r3), r4
	
	l.addi r3, r0, 0x00
	l.mtspr r0, r3, SPR_PICSR
	l.rfe
	l.nop
	
		
/* ---[ 0xf00: TMU exception ]----------------------------------------- */
    .org 0xf00
    .global _tmu_exception
_tmu_exception: 	
	l.movhi r0, 0
	l.addi r14, r14, 0x1
	l.nop
	//Clear counter
	l.mtspr r0, r0, SPR_TMU_CNT_L
	//Enable interrupts
	l.mfspr r3, r0, SPR_SR
	l.ori r3, r3, SPR_SR_IEE
	l.mtspr r0, r3, SPR_SR
	l.jal _enable_tmu_exception
	l.nop
	l.jr r10//Jump to next test
	l.nop

/* =================================================== [ text ] === */
	.section .text

/* =================================================== [ start ] === */	

	.global _start
_start:	
	l.jal _cache_init //Found in cache.S part of  or1200 driver
	l.nop
	
	// Kick off test
	l.jal   _main
	l.nop
	
/* =================================================== [ main ] === */	

_main:
	l.addi r12, r0, 0x0
	l.addi r13, r0, 0x000000f0
	l.addi r14, r0, 0x0
	l.addi r15, r0, 0x000000a0
	l.nop

/* =================================================== [ setup ] === */	
	
_setup_replenishment:
	//Set the replnishment registers
	l.addi r4, r0, 0x0
	l.movhi r5, hi(0x00005000)
	l.ori r5, r5, lo(0x00005000)
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_REPCMP_H
	l.mtspr r0, r5, SPR_TMU_REPCMP_L

_setup_interrupt_limits:
	l.addi r5, r0, 0x10
	//Write to compare register
	l.mtspr r0, r5, SPR_TMU_INTRCMP_BASE+3
	l.mtspr r0, r5, SPR_TMU_INTRCMP_BASE+4

_setup_pic:
	l.addi r4, r0, 0x1f
	l.mtspr r0, r4, SPR_PICMR
	
_setup_intgens:
	//Intgen 0
	l.movhi	r3, hi(0xa0000000)
	l.ori r3, r3, lo(0xa0000000)
	l.addi r4, r0, 0x00ff
	l.sw 0(r3), r4
	//Integen 1
	l.movhi	r3, hi(0xa1000000)
	l.ori r3, r3, lo(0xa1000000)
	l.addi r4, r0, 0x00af
	l.sw 0(r3), r4

/* =================================================== [ test code ] === */
	.global _standard1
	.global _standard2
	.global _inval
	.global _readsr
	.global _startstop
	.global _stopread
	.global _stop_test
	.global _testdisable

_test_main:
	l.jal _enable_interrupts
	l.nop

_standard1:
	/*
	*	Write compare and start count
	*/
	l.jal _enable_tmu_exception
	l.nop	
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x0f00
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//load address of next test into r10
	.global _standard2
	l.movhi r10, hi(_standard2)
	l.ori	r10, r10, lo(_standard2)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.j _wait
	l.nop 0x0

_standard2:
	/*
	*	Write compare and count, start count
	*	Should count for two cycles, no invalidation bit should be set
	*/
	l.jal _enable_tmu_exception
	l.nop
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x0f12
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//Write to count register
	l.addi r5, r0, 0x10
	l.mtspr r0, r4, SPR_TMU_CNT_H
	l.mtspr r0, r5, SPR_TMU_CNT_L
	//load address of next test into r10
	.global _inval
	l.movhi r10, hi(_inval)
	l.ori	r10, r10, lo(_inval)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.j _wait
	l.nop 0x0

_inval:
	/*
	*	Write compare without writing or clearing count, read SR
	*	Invalidation bit should be set
	*	Stop and clear count afterwards
	*/	
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x0a12
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//load address of next test into r10
	.global _readsr
	l.movhi r10, hi(_readsr)
	l.ori	r10, r10, lo(_readsr)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x900
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf -3
	l.nop
	//Stop timer
	l.addi r4, r0, SPR_TMU_CTRL_STOP
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.jr r10
	l.nop

_readsr:
	/*
	*	Write compare and start count, read SR while counting
	*/
	l.jal _enable_tmu_exception
	l.nop
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x600
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//load address of next test into r10
	.global _startstop
	l.movhi r10, hi(_startstop)
	l.ori	r10, r10, lo(_startstop)
	//Clear count and start timer
	l.addi r4, r0, SPR_TMU_CTRL_RESTART|SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x100
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf (_readsr+0x2c)
	l.nop
	//Read SR
	l.mfspr r0, r4, SPR_TMU_STATUS
	l.j _wait
	l.nop 0x0
	
_startstop:
	/*
	*	Write count and compare, start count, wait then stop and start again
	*	Should continue counting after second start
	*/
	l.jal _enable_tmu_exception
	l.nop
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x1000
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//Write to count register
	l.addi r5, r0, 0x10
	l.mtspr r0, r4, SPR_TMU_CNT_H
	l.mtspr r0, r5, SPR_TMU_CNT_L
	//load address of next test into r10
	.global _stopread
	l.movhi r10, hi(_stopread)
	l.ori	r10, r10, lo(_stopread)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x5
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf -3//(_startstop+0x38)
	l.nop
	//Stop timer
	l.addi r4, r0, SPR_TMU_CTRL_STOP
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x10
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf -3//(_startstop+0x4c)
	l.nop
	//Start tmu again
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.j _wait
	l.nop 0x0
	
_stopread:
	/*
	*	Start counter. Stop it then read registers
	*/
	l.jal _enable_tmu_exception
	l.nop
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x600
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//Write to count register
	l.addi r5, r0, 0x10
	l.mtspr r0, r4, SPR_TMU_CNT_H
	l.mtspr r0, r5, SPR_TMU_CNT_L
	//load address of next test into r10
	.global _wait
	l.movhi r10, hi(_wait)
	l.ori	r10, r10, lo(_wait)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x10
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf -3//(_stopread+0x38)
	l.nop
	//Stop timer
	l.addi r4, r0, SPR_TMU_CTRL_STOP
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Read registers and store them in memory
	l.addi r3, r0, 0x7000
	l.mfspr r0, r4, SPR_TMU_CNT_H
	l.sw 0x0(r3), r4
	l.mfspr r0, r5, SPR_TMU_CNT_L
	l.sw 0x4(r3), r4
	l.mfspr r0, r6, SPR_TMU_CMP_H
	l.sw 0x8(r3), r4
	l.mfspr r0, r7, SPR_TMU_CMP_L
	l.sw 0xc(r3), r4
	l.mfspr r0, r8, SPR_TMU_CTRL
	l.sw 0x10(r3), r4
	//Start timer again
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.j _wait
	l.nop
	
/* =================================================== [ waiting loop ] === */
		
_wait:
	l.sfgeu r12, r13
	l.bf 0x5 //Jump to stop test
	l.sfgeu r14, r15
	l.bf 0x3 //Jump to stop test
	l.j _wait
	l.nop
	l.j _stop_test
	l.nop

_stop_test:
	l.movhi r3, 0x8000
	l.ori r3, r3, 0x000d
	l.nop 0x2
	l.movhi r3, 0x0
	l.nop 0x1
	l.nop

_enable_tmu_exception:
	l.mfspr r3, r0, SPR_SR
	l.movhi r4, hi(SPR_SR_CEE)
	l.ori r4, r4, lo(SPR_SR_CEE)
	l.or r3, r3, r4
	l.mtspr r0, r3, SPR_SR
	l.jr r9
	l.nop

_enable_interrupts:
	l.mfspr r3, r0, SPR_SR
	l.ori r3, r3, SPR_SR_IEE
	l.mtspr r0, r3, SPR_SR
	l.jr r9
	l.nop

_disable_interrupts:
	l.mfspr r3, r0, SPR_SR
	l.andi r3, r3, ~SPR_SR_IEE
	l.mtspr r0, r3, SPR_SR
	l.jr r9
	l.nop

_reset_integen0:
	//Clear intgen 0
	l.movhi	r5, hi(0xa0000004)
	l.ori r5, r5, lo(0xa0000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Reset timer 0
	l.movhi	r3, hi(0xa0000000)
	l.ori r3, r3, lo(0xa0000000)
	l.addi r4, r0, 0x00ff
	l.sw 0(r3), r4
	l.j _wait
	l.nop
	
_reset_intgen1:
	//Clear intgen 1
	l.movhi	r5, hi(0xa1000004)
	l.ori r5, r5, lo(0xa1000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Integen 1
	l.movhi	r3, hi(0xa1000000)
	l.ori r3, r3, lo(0xa1000000)
	l.addi r4, r0, 0x00af
	l.sw 0(r3), r4
	l.j _wait
	l.nop	
		
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
