/*
	Test the TMU


*/

#include "spr-defs.h"
#include "board.h"
#include "or1200-defines.h"

/* =================================================== [ exceptions ] === */
	.section .vectors, "ax"


/* ---[ 0x100: RESET exception ]----------------------------------------- */
        .org 0x100 	
	l.movhi r0, 0
	/* Clear status register */
	l.ori r1, r0, SPR_SR_SM
	l.mtspr r0, r1, SPR_SR
	/* Clear timer  */
	l.mtspr r0, r0, SPR_TTMR

	/* Jump to program initialisation code */
	.global _start
	l.movhi r4, hi(_start)
	l.ori r4, r4, lo(_start)
	l.jr    r4
	l.nop
	
/* ---[ 0xf00: TMU exception ]----------------------------------------- */
        .org 0xf00 	
	l.nop
	l.mtspr r0, r0, SPR_TMU_CNT_L
	l.jr r10//Jump to next test
	l.nop

/* =================================================== [ text ] === */
	.section .text

/* =================================================== [ start ] === */	

	.global _start
_start:	
	l.jal _cache_init //Found in cache.S part of  or1200 driver
	l.nop
	
	// Kick off test
	l.jal   _main
	l.nop
	
/* =================================================== [ main ] === */	
	.global _main
	.global _standard1
	.global _standard2
	.global _inval
	.global _readsr
	.global _startstop
	.global _stopread
	.global _stop_test
	
_main:
	l.nop
	
	
_standard1:
	/*
	*	Write compare and start count
	*/
	l.jal _enable_tmu_exception
	l.nop	
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x10
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//load address of next test into r10
	.global _standard2
	l.movhi r10, hi(_standard2)
	l.ori	r10, r10, lo(_standard2)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.j _wait_tmu
	l.nop 0x0

_standard2:	
	/*
	*	Write compare and count, start count
	*	Should count for 30 cycles, no invalidation bit should be set
	*/
	l.jal _enable_tmu_exception
	l.nop
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x40
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//Write to count register
	l.addi r5, r0, 0x10
	l.mtspr r0, r4, SPR_TMU_CNT_H
	l.mtspr r0, r5, SPR_TMU_CNT_L
	//load address of next test into r10
	.global _inval
	l.movhi r10, hi(_inval)
	l.ori	r10, r10, lo(_inval)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.j _wait_tmu
	l.nop 0x0

_inval:
	/*
	*	Write compare without writing or clearing count, read SR
	*	Invalidation bit should be set
	*	Stop and clear count afterwards
	*/
	l.jal _enable_tmu_exception
	l.nop	
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x12
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//load address of next test into r10
	.global _readsr
	l.movhi r10, hi(_readsr)
	l.ori	r10, r10, lo(_readsr)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.j _wait_count
	l.nop

_readsr:
	/*
	*	Write compare and start count, read SR while counting
	*/
	l.jal _enable_tmu_exception
	l.nop
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x100
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//load address of next test into r10
	.global _startstop
	l.movhi r10, hi(_startstop)
	l.ori	r10, r10, lo(_startstop)
	//Clear count and start timer
	l.addi r4, r0, SPR_TMU_CTRL_RESTART|SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x10
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf (_readsr+0x2c)
	l.nop
	//Read SR
	l.mfspr r4, r0, SPR_TMU_STATUS
	l.j _wait_tmu
	l.nop 0x0
	
_startstop:
	/*
	*	Write count and compare, start count, wait then stop and start again
	*	Should continue counting after second start
	*/
	l.jal _enable_tmu_exception
	l.nop
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x50
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//Write to count register
	l.addi r5, r0, 0x10
	l.mtspr r0, r4, SPR_TMU_CNT_H
	l.mtspr r0, r5, SPR_TMU_CNT_L
	//load address of next test into r10
	.global _stopread
	l.movhi r10, hi(_stopread)
	l.ori	r10, r10, lo(_stopread)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x10
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf (_startstop+0x38)
	l.nop
	//Stop timer
	l.addi r4, r0, SPR_TMU_CTRL_STOP
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x10
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf (_startstop+0x4c)
	l.nop
	//Start tmu again
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.j _wait_tmu
	l.nop 0x0
	
_stopread:
	/*
	*	Start counter. Stop it then read registers
	*/
	l.jal _enable_tmu_exception
	l.nop
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x60
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	//Write to count register
	l.addi r5, r0, 0x10
	l.mtspr r0, r4, SPR_TMU_CNT_H
	l.mtspr r0, r5, SPR_TMU_CNT_L
	//load address of next test into r10
	.global _testdisable
	l.movhi r10, hi(_stop_test)
	l.ori	r10, r10, lo(_stop_test)
	//Start timer
	l.addi r4, r0, SPR_TMU_CTRL_START
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Loop for some time
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x10
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf (_stopread+0x38)
	l.nop
	//Stop timer
	l.addi r4, r0, SPR_TMU_CTRL_STOP
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Read registers and store them in memory
	l.addi r3, r0, 0x7000
	l.mfspr r0, r4, SPR_TMU_CNT_H
	l.sw 0x0(r3), r4
	l.mfspr r0, r5, SPR_TMU_CNT_L
	l.sw 0x4(r3), r4
	l.mfspr r0, r6, SPR_TMU_CMP_H
	l.sw 0x8(r3), r4
	l.mfspr r0, r7, SPR_TMU_CMP_L
	l.sw 0xc(r3), r4
	l.mfspr r0, r8, SPR_TMU_CTRL
	l.sw 0x10(r3), r4
	l.j _stop_test
	l.nop
	
_wait_tmu:
	l.nop
	l.nop
	l.j _wait_tmu

_wait_count:
	l.addi r3, r0, 0x0
	l.addi r4, r0, 0x100
	l.nop
	l.addi r3, r3, 0x1
	l.sfeq r3, r4
	l.bnf (_wait_count+0xc)
	l.nop
	//Stop timer
	l.addi r4, r0, SPR_TMU_CTRL_STOP
	l.mtspr r0, r4, SPR_TMU_CTRL
	//Clear values
	l.addi r4, r0, SPR_TMU_CTRL_CLEAR
	l.mtspr r0, r4, SPR_TMU_CTRL
	l.jr r10
	l.nop

_enable_tmu_exception:
	l.mfspr r3, r0, SPR_SR
	l.movhi r4, hi(SPR_SR_CEE)
	l.ori r4, r4, lo(SPR_SR_CEE)
	l.or r3, r3, r4
	l.mtspr r0, r3, SPR_SR
	l.jr r9
	l.nop
	
_stop_test:
	l.movhi r3, 0x8000
	l.ori r3, r3, 0x000d
	l.nop 0x2
	l.movhi r3, 0x0
	l.nop 0x1
	l.nop
	
/*	
_writecomp: //Set all GPRs to 0
	//Load the compare hi and low values into r4 and r5
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x10
	//Write to compare register
	l.mtspr r0, r4, SPR_TMU_CMP_H
	l.mtspr r0, r5, SPR_TMU_CMP_L
	
_writecount:
	//Load the count hi and low values into r4 and r5
	l.addi r4, r0, 0x0
	l.addi r5, r0, 0x0
	l.mtspr r0, r4, SPR_TMU_CNT_H
	l.mtspr r0, r5, SPR_TMU_CNT_L
	
_starttmu:
	//Load the SR value into r4
	l.addi r4, r0, 0x1
	l.mtspr r0, r4, SPR_TMU_CTRL
	
_initcount:
	l.addi r3, r0, 0x1
	l.nop 0x0
	
_wait_exception:
	l.addi r3, r3, 0x2
	l.j _wait_exception
	l.nop 0x0
*/	

	
		
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
