/*
	Test the full operation of the TMU with two interrupt sources, and three running tasks
	The three running tasks will be run on a round-robin basis with the same proirity
	Each task has a critiacl section in which interrupts and ticks are disable,
	and it has a 10% chance of exeding its budget in the critical section



*/

#include "spr-defs.h"
#include "board.h"
#include "or1200-defines.h"
//#include "lib-utils.h"


/* =================================================== [ exceptions ] === */
	.section .vectors, "ax"


/* ---[ 0x100: RESET exception ]----------------------------------------- */
    .org 0x100 	
	l.movhi r0, 0
	l.movhi r1, 0
	l.movhi r2, 0
	l.movhi r3, 0
	l.movhi r4, 0
	l.movhi r5, 0
	l.movhi r6, 0
	l.movhi r7, 0
	l.movhi r8, 0
	l.movhi r9, 0
	l.movhi r10, 0
	l.movhi r11, 0
	l.movhi r12, 0
	l.movhi r13, 0
	l.movhi r14, 0
	l.movhi r15, 0
	l.movhi r16, 0
	l.movhi r17, 0
	l.movhi r18, 0
	l.movhi r19, 0
	l.movhi r20, 0
	l.movhi r21, 0
	l.movhi r22, 0
	l.movhi r23, 0
	l.movhi r24, 0
	l.movhi r25, 0
	l.movhi r26, 0
	l.movhi r27, 0
	l.movhi r28, 0
	l.movhi r29, 0
	l.movhi r30, 0
	l.movhi r31, 0
	/* Clear status register */
	l.ori r1, r0, SPR_SR_SM
	l.mtspr r0, r1, SPR_SR
	/* Clear timer  */
	l.mtspr r0, r0, SPR_TTMR
	

	/* Jump to program initialisation code */
	.global _start
	l.movhi r4, hi(_start)
	l.ori r4, r4, lo(_start)
	l.jr    r4
	l.nop

/* --- [ tick interrupt ]-------------------------- */
	.org 0x500
	.global _tick_handler
_tick_handler:
		

/* ---[ 0x800: external interrupt exception ]-------------------------- */
    .org 0x800
    .global _exint_exception
_exint_exception:
	l.movhi r0, 0
	l.addi r12, r12, 0x1
	l.nop
	l.mfspr r3, r0, SPR_PICSR
	//Mask out intgen 0 as source
	l.andi r4, r3, 0x8
	l.sfeq r4, r0
	l.bf 0x9//Branc to next source
	//Clear intgen 0
	l.movhi	r5, hi(0xa0000004)
	l.ori r5, r5, lo(0xa0000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Reset timer 0
	l.movhi	r3, hi(0xa0000000)
	l.ori r3, r3, lo(0xa0000000)
	l.addi r4, r0, 0x00ff
	l.sw 0(r3), r4
	
	//Mask out intgen 1 as source
	l.andi r4, r3, 0x10
	l.sfeq r4, r0
	l.bf 0x9 //Branch to return
	//Clear intgen 1
	l.movhi	r5, hi(0xa1000004)
	l.ori r5, r5, lo(0xa1000004)
	l.addi r4, r0, 0x00
	l.sw 0(r5), r4
	//Integen 1
	l.movhi	r3, hi(0xa1000000)
	l.ori r3, r3, lo(0xa1000000)
	l.addi r4, r0, 0x00af
	l.sw 0(r3), r4
	
	l.addi r3, r0, 0x00
	l.mtspr r0, r3, SPR_PICSR
	l.rfe
	l.nop
	
		
/* ---[ 0xf00: TMU exception ]----------------------------------------- */
    .org 0xf00
    .global _tmu_exception
_tmu_exception: 	
	l.movhi r0, 0
	l.addi r14, r14, 0x1
	l.nop
	//Enable interrupts
	l.mfspr r3, r0, SPR_SR
	l.ori r3, r3, SPR_SR_IEE
	l.mtspr r0, r3, SPR_SR
	l.jr r10//Jump to next test
	l.nop

/* =================================================== [ text ] === */
	.section .text

/* =================================================== [ start ] === */	

	.global _start
	.global _main
	
_start:	
	l.jal _cache_init //Found in cache.S part of  or1200 driver
	l.nop
	
	// Kick off test
	l.jal   _main
	l.nop

_main:
	
	l.nop

_enable_interrupts:
	l.jal cpu_enable_user_interrupts
	l.nop
	l.jal cpu_enable_timer
	l.nop
	
	l.jr r9
	l.nop
	
_disable_interrupts:
	l.jal cpu_disable_user_interrupts
	l.nop
	l.jal cpu_disable_timer
	l.nop

	l.jr r9
	l.nop
























