/*
	TMU-full test support functionality
*/

#include "cpu-utils.h"
#include "lib-utils.h"
#include "spr-defs.h"
#include "int.h"
#include "tmu.h"

#include "tmu-full.h"

#define TICK_PERIOD 100000

const int INTGEN_BASE[2] = {0xa0000000, 0xa1000000};
const unsigned long long task_limits[3] = {2*TICK_PERIOD, 1.5*TICK_PERIOD, 2.5*TICK_PERIOD};
const unsigned long long replenishment = 6*TICK_PERIOD;
const unsigned long intr_limits[2] = {5, 4};

unsigned long timer_ticks;
unsigned int lastTask;

#define INT0 0x03
#define INT1 0x04

void idle();

void enable_interrupts() {
	cpu_enable_ticks();
	cpu_enable_user_interrupts();
	cpu_enable_tmu_interrupts();
}
void disable_interrupts() {
	cpu_disable_ticks();
	cpu_disable_user_interrupts();
}

static void *interrupt0(void) {
	REG32(INTGEN_BASE[0] + 0x4) = 0x00000000;
	REG32(INTGEN_BASE[0]) = (rand()>>15);
	//report(0xa0000000);
}

static void *interrupt1(void) {
	REG32(INTGEN_BASE[1] + 0x4) = 0x00000000;
	REG32(INTGEN_BASE[1]) = (rand()>>15);
	//report(0xa1000000);
}

void tick_handler(void) {
	timer_ticks++;
	if (timer_ticks == 10) exit(0);
	tmu_stop();
	mtspr(SPR_TTMR, SPR_TTMR_IE | SPR_TTMR_RT | ((50000000/500) & SPR_TTMR_PERIOD));
	switch_task();
}

void tmu_except_handler(void) {
	tmu_stop();
	tmu_set_count(0x00000000);
	switch_task();
}

int main (void) {
	tmu_stop();
	timer_ticks = 0;
	lastTask = 2;
	cpu_init_timer();
	int_init();
	int_add(INT0, interrupt0, NULL);
	int_add(INT1, interrupt1, NULL);
	tmu_set_interrupt_rep_period(replenishment);
	tmu_set_interrupt_compare(INT0, intr_limits[0]);
	tmu_set_interrupt_compare(INT1, intr_limits[1]);
	enable_interrupts();
	REG32(INTGEN_BASE[0]) = (rand()>>12);
	REG32(INTGEN_BASE[1]) = (rand()>>12);
	switch_task();
	while(1) {
		;;
	}
}

void switch_task(void) {
	lastTask++;
	if ((lastTask%3) == 0) {
		lastTask = 0;
		tmu_set_compare(task_limits[0]);
		tmu_restart();
		tmu_start();
		enable_interrupts();
		task1();
	} else if ((lastTask%3) == 1) {
		lastTask = 1;
		tmu_set_compare(task_limits[1]);
		tmu_restart();
		tmu_start();
		enable_interrupts();
		task2();
	} else if ((lastTask%3) == 2) {
		lastTask = 2;
		tmu_set_compare(task_limits[2]);
		tmu_restart();
		tmu_start();
		enable_interrupts();
		task3();
	} else {
		;;
	}
}

void task1(void) {
	unsigned int fail;
	int count = 0;
	for (;;) {
		for (count = 0; count < 1000; count++) {
			static int m, n, o;
			m=1;
			n=2;
			o=m*n;
		}
		//Critical section
		
		fail = (rand()>>24);
		disable_interrupts();
		if (fail < 25) {
			//Use long time
			for (;;) {
				int a = 1;
				int b = a;
				int c = a + b;
			}
		} else {
			report(0x10000000);
		}
		enable_interrupts();
	}
}

void task2(void) {
	unsigned int fail;
	int count = 0;
	for (;;) {
		for (count = 0; count < 1000; count++) {
			static int m, n, o;
			m=1;
			n=2;
			o=m*n;
		}
		//Critical section
		fail = (rand()>>24);
		disable_interrupts();
		if (fail < 25) {
			//Use long time
			for (;;) {
				int a = 1;
				int b = a;
				int c = a + b;
			}
		} else {
			report(0x20000000);
		}
		enable_interrupts();
	}
}

void task3(void) {
	unsigned int fail;
	int count = 0;
	for (;;) {
		for (count = 0; count < 1000; count++) {
			static int m, n, o;
			m=1;
			n=2;
			o=m*n;
		}
		//Critical section
		fail = (rand()>>24);
		disable_interrupts();
		if (fail < 25) {
			//Use long time
			for (;;) {
				int a = 1;
				int b = a;
				int c = a + b;
			}
		} else {
			report(0x30000000);
		}
		enable_interrupts();
	}
}

void idle() {
	report(0xf0000000);
	enable_interrupts();
	for (;;) {;;}
}






