

`include "or1200_defines.v"
`include "bench_defines.v"




module or1200_tmu_tb ();

reg 			clk;
reg 			rst;
	
	// SPR interface
reg 	[31:0]	spr_addr;
reg 	[31:0]	spr_dat_i;
reg 	[31:0]	spr_cs;
reg 			spr_write; 
wire 	[31:0]	spr_dat_o;
	
reg		[`OR1200_PIC_INTS-1:0] pic_ints;
wire	[`OR1200_PIC_INTS-1:0] pic_ints_masked;	

	//exception interface
wire			sig_tmu;
reg				except_started;
reg				ex_freeze;

integer			i;

initial begin
	$dumpfile("wave_output.vcd");
	$dumpvars(0, or1200_tmu_tb);
end
	

initial begin
	clk 		<= 0;
	rst 		<= !`OR1200_RST_VALUE;
	spr_addr 	<= 0;
	spr_dat_i 	<= 0;
	spr_cs 		<= 0;
	spr_write 	<= 0;
	pic_ints	<= 0;
	except_started <= 0;
	ex_freeze	<= 0;
	i 			<= 0;
end

always begin
	#(10) clk <= ~clk;
end

initial begin
	repeat (10) @(negedge clk);
	rst <= `OR1200_RST_VALUE;
	repeat (10) @(negedge clk);
	rst <= !`OR1200_RST_VALUE;
end


//Stimuli
initial begin
	@(posedge rst);
	@(negedge rst);
	repeat (10) @(posedge clk);
`ifdef SIMPLE
	/*
	*	Write compare and start count
	*/
	$display("\t### Start simple test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	//Wait for exception
	fork : wait_simple
		begin
			#40000;//400 clock cycles
			$display("Test failed");
			$finish;
		end
		begin
			@(posedge sig_tmu);
			disable wait_simple;
		end
	join
	//Clear count
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t### Simple test done ###\n");
	repeat(10) @(posedge clk);
`endif
`ifdef SIMPLE2
	/*
	*	Write compare and count, start count
	*	Should count for two cycles, no invalidation bit should be set
	*/
	$display("\t### Start simple2 test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0012;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	//Wait for exception
	fork : wait_simple2
		begin
			#40000;//400 clock cycles
			$display("Test failed");
			$finish;
		end
		begin
			@(posedge sig_tmu);
			disable wait_simple2;
		end
	join
	//Clear count
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t### Simple2 test done ###\n");
	repeat(10) @(posedge clk);
`endif
`ifdef ZERO
	/*
	*	Write compare and count to zero, start count
	*	Should count, but not generate exception
	*/
	$display("\t### Start zero test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	//Wait for exception
	fork : wait_zero
		begin
			#4000;//40 clock cycles
			disable wait_zero;
		end
		begin
			@(posedge sig_tmu);
			$display("Test failed");
			$finish;
		end
	join
	//Stop timer
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_STATUS};
	spr_dat_i							<= 32'h0000_0002;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write			
	$display("\t###Zero test done###\n");
	repeat(10) @(posedge clk);
`endif
`ifdef NOEXCEPT
	/*
	*	Write compare and count, disable exception and start count
	*	Should count forever
	*/
	$display("\t### Start no exception test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0012;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk)
	//Start timer disable exceptions
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_STATUS};
	spr_dat_i							<= 32'h0000_0201;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	//Wait for exception
	fork : wait_noexcept
		begin
			#4000;//40 clock cycles
			disable wait_noexcept;
		end
		begin
			@(posedge sig_tmu);
			$display("Test failed");
			$finish;
		end
	join
	//Stop and enable exceptions
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_STATUS};
	spr_dat_i							<= 32'h0000_0102;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t### No exception test done ###\n");
	repeat(10) @(posedge clk);
`endif
`ifdef TESTREADSR
	/*
	*	Write compare and start count, read SR while counting
	*/
	$display("\t### Start read sr test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0100;
	@(posedge clk)
	//Restart count and start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0005;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	//Wait for exception
	fork : wait_readsr
		begin
			#40000;//400 clock cycles
			$display("Test failed timeout");
			$finish;
		end
		begin
			#4000;//40 clock cycles
			spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
			spr_addr[31:16]				 		<= 16'h0000;
			spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
			spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_STATUS};
			if (spr_dat_o == 0) begin
				$display("Test failed data");
				$finish;
			end
		end
		begin
			@(posedge sig_tmu);
			disable wait_readsr;
		end
	join
	//Clear count
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t###Standard test with read during count done###\n");
	repeat(10) @(posedge clk);
`endif
`ifdef TESTFREEZE	
	/*
	*	Write compare and start count, freeze CPU
	*	Should continue counting during freeze
	*/
	$display("\t### Start freeze test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0100;
	@(posedge clk)
	//Clear count and start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0005;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(20) @(posedge clk);
	ex_freeze							<= 1;
	repeat(30) @(posedge clk);
	ex_freeze							<= 0;
	//Wait for exception
	fork : wait_freeze
		begin
			#40000;//400 clock cycles
			$display("Test failed");
			$finish;
		end
		begin
			@(posedge sig_tmu);
			disable wait_freeze;
		end
		begin
			#2000
			ex_freeze <= 1;
			#200
			spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
			spr_addr[31:16]				 		<= 16'h0000;
			spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
			spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_STATUS};
			if (spr_dat_o[0] == 0) begin
				$display("Test failed data");
				$finish;
			end
			#2000
			ex_freeze <= 0;
		end
	join
	//Clear count
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t###Standard test with freeze done###\n");
	repeat(10) @(posedge clk);
`endif
`ifdef TESTNOFREEZE	
	/*
	*	Write compare and start count, set to stop count during freeze, freeze CPU
	*	Should not continue counting during freeze
	*/
	$display("\t### Start no freeze test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0100;
	@(posedge clk)
	//Clear count and start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_1005;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	fork : wait_nofreeze
		begin
			#40000;//400 clock cycles
			$display("Test failed");
			$finish;
		end
		begin
			@(posedge sig_tmu);
			disable wait_nofreeze;
		end
		begin
			#2000
			ex_freeze <= 1;
			#200
			spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
			spr_addr[31:16]				 		<= 16'h0000;
			spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
			spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_STATUS};
			if (spr_dat_o[`OR1200_TMU_SR_SUS] == 0) begin
				$display("Test failed data");
				$finish;
			end
			#200
				spr_cs								<= 0;
				spr_addr							<= 0;
				spr_dat_i							<= 0;
				spr_write							<= 0;
			#2000
			ex_freeze <= 0;
		end
	join
	//Clear count
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	//Enable freeze count
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_2000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t###Standard test without freeze-count done###\n");
	repeat(10) @(posedge clk);
`endif
`ifdef TESTRST	
	/*
	*	Write count and compare, start count then reset
	*	Should not continue couting after reset
	*/
	$display("\t### Start reset test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0090;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	@(posedge clk);
	rst									<= 1;
	repeat(10) @(posedge clk);
	rst									<= 0;
	repeat(10) @(posedge clk);
	fork : timed_wait
		begin
			#4000;//40 clock cycles
			disable timed_wait;
		end
		begin
			@(posedge sig_tmu);
			$display("Test failed");
			$finish;
		end
	join
	$display("\t###Test with reset done###\n");
`endif
`ifdef TESTSTARTSTOP	
	/*
	*	Write count and compare, start count, wait then stop and start again
	*	Should continue counting after second start
	*/
	$display("\t### Start start/stop test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0050;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(20) @(posedge clk);
	//Stop timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0002;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(20) @(posedge clk);
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	
	//Wait for exception
	fork : wait_startstop
		begin
			#40000;//400 clock cycles
			$display("Test failed");
			$finish;
		end
		begin
			@(posedge sig_tmu);
			disable wait_startstop;
		end
	join
	//Clear count
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t###Standard test with start and stop done###\n");
	repeat(10) @(posedge clk);
`endif
`ifdef TESTSTOPREAD	
	/*
	*	Start counter. Stop it then read Count registers
	*/
	
	$display("\t### Start stop-then-read test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0050;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(20) @(posedge clk);
	//Stop timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0002;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(2) @(posedge clk);
	//Read count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	if (spr_dat_o != 0) begin
		$display("Test failed");
		$finish;
	end
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(10) @(posedge clk);
	$display("\t###Read COUNT registers###\n");
`endif
`ifdef TESTRND	
	/*
	*	Apply random stimuli
	*	Nothing should happen
	*/
	$display("\t### Start random stimuli ###\n");
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
		fork : wait_random
		begin
			#4000;//40 clock cycles
			disable wait_random;
		end
		begin
			while (sig_tmu != 1) begin
				@(posedge clk);
				spr_addr							<= $random;
				spr_dat_i							<= $random;
			end
		end
		begin
			@(posedge sig_tmu);
			$display("Test failed");
			$finish;
		end
	join
	
	$display("\t###Random stimuli test done###\n");
`endif
`ifdef TESTCOUNTRND
	/*
	*	Write count and compare, apply random stimuli
	*	Should contiue to count and generate exception
	*/
	$display("\t### Start random stimuli with count ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0050;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0006;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	fork : timed_cntrnd
		begin
			#400000;//400 clock cycles
			$display("Test failed");
			$finish;
		end
		begin
			while (sig_tmu != 1) begin
				@(posedge clk);
				spr_addr							<= $random;
				spr_dat_i							<= $random;
			end
		end
		begin
			@(posedge sig_tmu);
			disable timed_cntrnd;
		end
	join
	//Clear count
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t###Random stimuli and count done###\n");
`endif
`ifdef TESTEXCYES
	/*
	*	Write compare and count register, start counting
	*	During operation, set except_started
	*	The module should continue counting when exeption is started
	*/
	$display("\t### Start count during exception test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0050;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(20) @(posedge clk);
	except_started						<= 1;
	@(posedge clk);
	except_started						<= 0;
	fork : wait_exyes
		begin
			#40000;//400 clock cycles
			$display("Test failed");
			$finish;
		end
		begin
			@(posedge sig_tmu);
			disable wait_exyes;
		end
	join
	//Clear count
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(20) @(posedge clk);
`endif
`ifdef TESTEXCNO
	/*
	*	Write compare and count, and set the module to not count during exceptions
	*	Start timer, and set except_started
	*	Module should stop counting
	*/
	$display("\t### Start no count during exception test ###\n");
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_0050;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	//Set no count during exception and start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0801;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(20) @(posedge clk);
	except_started						<= 1;
	@(posedge clk);
	except_started						<= 0;
	
	fork : wait_exno
		begin
			#4000;//40 clock cycles
			disable wait_exno;
		end
		begin
			@(negedge except_started);
			@(posedge clk);
			@(posedge clk);
			spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
			spr_addr[31:16]				 		<= 16'h0000;
			spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
			spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_STATUS};
			if (spr_dat_o[`OR1200_TMU_SR_R] == 1) begin
				$display("Test failed data");
				$finish;
			end
			#200
			spr_cs								<= 0;
			spr_addr							<= 0;
			spr_dat_i							<= 0;
			spr_write							<= 0;
		end
		begin
			@(posedge sig_tmu);
			$display("Test failed exception");
			$finish;
		end
	join
	@(posedge clk)
	//Set count during exception
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0400;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t### No count during exception test done###\n");
	repeat(20) @(posedge clk);
`endif
`ifdef REPTEST
	/*
	*	Write replenish compare register
	*	Module should restart this timer when it runs out
	*/
	$display("\t### Replenish counter test started ###\n");
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	@(posedge clk)
	@(posedge dut.replenish);
	repeat (100) @(posedge clk);
	$display("\t### Replenish counter test done ###\n");
`endif
`ifdef NOMASK
	/*
	*	Write replenish and an interrupt compare register
	*	Set the interrupt some times, less than budget
	*	The count should reset at replenishment time
	*/
	$display("\t### No masking test started ###\n");
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_LO};
	spr_dat_i							<= 32'h0000_0100;
	@(posedge clk)
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_INTRCOMP};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat (8) begin
		@(posedge clk);
		pic_ints[0]							<= 1'b1;
		@(posedge clk);
		pic_ints[0]							<= 1'b0;
		repeat(5) @(posedge clk);
	end
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_PIC_MASK};
	if (spr_dat_o[`OR1200_TMU_SR_R] != 0) begin
		$display("Test failed data");
		$finish;
	end
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat (100) @(posedge clk);
	repeat (5) begin
		@(posedge clk);
		pic_ints[0]							<= 1'b1;
		@(posedge clk);
		pic_ints[0]							<= 1'b0;
		repeat(5) @(posedge clk);
	end
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_PIC_MASK};
	if (spr_dat_o[`OR1200_TMU_SR_R] != 0) begin
		$display("Test failed data");
		$finish;
	end
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t### No masking test  done ###\n");
`endif
`ifdef MASKTEST
	/*
	*	Write replenish and an interrupt compare register
	*	Set the interrupt some times, more than budget
	*	The corresponding interrupt mask should be set
	*	The count should reset at replenishment time
	*/
	$display("\t### Mask test started ###\n");
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_LO};
	spr_dat_i							<= 32'h0000_0050;
	@(posedge clk)
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_INTRCOMP};
	spr_dat_i							<= 32'h0000_0005;
	@(posedge clk)
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat (8) begin
		@(posedge clk);
		pic_ints[0]							<= 1'b1;
		@(posedge clk);
		pic_ints[0]							<= 1'b0;
		repeat(5) @(posedge clk);
	end
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_PIC_MASK};
	if (spr_dat_o == 0) begin
		$display("Test failed data");
		$finish;
	end
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat (40) @(posedge clk);
	repeat (8) begin
		@(posedge clk);
		pic_ints[0]							<= 1'b1;
		@(posedge clk);
		pic_ints[0]							<= 1'b0;
		repeat(5) @(posedge clk);
	end
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_PIC_MASK};
	if (spr_dat_o == 0) begin
		$display("Test failed data");
		$finish;
	end
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t### Mask test done ###\n");
`endif
`ifdef MASKDISABLE
	/*
	*	Load values into replenish and interrupt compare registers, disabøe interrupt counting
	*	Apply random stimuli to interrupts
	*/
	$display("\t### Mask disable test started ###\n");
	@(posedge clk);
	rst <=1;
	@(posedge clk);
	rst <=0;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_LO};
	spr_dat_i							<= 32'h0000_0050;
	for (i = 0; i < `OR1200_PIC_INTS; i = i + 1) begin	
	@(posedge clk)
		spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
		spr_write							<= 1;
		spr_addr[31:16]				 		<= 16'h0000;
		spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
		spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_INTRCOMP+i};
		spr_dat_i							<= 32'h0000_0005;
	end
	@(posedge clk)
	//Disable interrupt counting
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_8000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;	
	repeat (100) begin
		@(posedge clk);
		pic_ints 						<= $random;
		if (pic_ints != pic_ints_masked) begin
			$display("Test failed");
			$finish;
		end
	end
	//Disable interrupt counting
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_4000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t### Mask disable test done ###\n");
`endif
`ifdef INTRTESTRND
	/*
	*	Load values into replenish and interrupt compare registers
	*	Apply random stimuli to interrupts
	*/
	$display("\t### Random interrupt counter test started ###\n");
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_LO};
	spr_dat_i							<= 32'h0000_0050;
	for (i = 0; i < `OR1200_PIC_INTS; i = i + 1) begin	
	@(posedge clk)
		spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
		spr_write							<= 1;
		spr_addr[31:16]				 		<= 16'h0000;
		spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
		spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_INTRCOMP+i};
		spr_dat_i							<= 32'h0000_0005;
	end
	@(posedge clk)
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;	
	repeat (500) begin
		@(posedge clk);
		pic_ints 						<= $random;
	end
	$display("\t### Random interrupt counter test done ###\n");
`endif
`ifdef TESTFULL
	/*
	*	Load values into replenish and interrupt compare registers
	*	Load values into count and compare registers, start count
	*	Apply random stimuli to interrupts
	*/
	$display("\t### Full test started ###\n");
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_REPCOMP_LO};
	spr_dat_i							<= 32'h0000_0020;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(10) @(posedge clk);
	for (i = 0; i < `OR1200_PIC_INTS; i = i + 1) begin	
	@(posedge clk)
		spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
		spr_write							<= 1;
		spr_addr[31:16]				 		<= 16'h0000;
		spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
		spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_INTRCOMP+i};
		spr_dat_i							<= 32'h0000_0005;
	end
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	repeat(10) @(posedge clk);
	@(posedge clk)
	//Write compare value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COMPARE_LO};
	spr_dat_i							<= 32'h0000_070;
	@(posedge clk)
	//Write count value
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_HI};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0010;
	@(posedge clk)
	//Start timer
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_CTRL};
	spr_dat_i							<= 32'h0000_0001;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	fork : wait_full
		begin
			#400000;//400 clock cycles
			$display("Test failed");
			$finish;
		end
		begin
			@(posedge sig_tmu);
			disable wait_full;
		end
		begin
			while (sig_tmu != 1) begin
				@(posedge clk);
				pic_ints							<= $random;
			end
		end
	join
	//Clear count
	@(posedge clk);
	spr_cs[`OR1200_SPR_GROUP_TMU]		<= 1;
	spr_write							<= 1;
	spr_addr[31:16]				 		<= 16'h0000;
	spr_addr[`OR1200_SPR_GROUP_BITS]	<= `OR1200_SPR_GROUP_TMU;
	spr_addr[`OR1200_SPR_OFS_BITS]		<= {8'h00, `OR1200_TMU_OFS_COUNT_LO};
	spr_dat_i							<= 32'h0000_0000;
	@(posedge clk);
	spr_cs								<= 0;
	spr_addr							<= 0;
	spr_dat_i							<= 0;
	spr_write							<= 0;
	$display("\t### Full test done ###\n");
`endif
	repeat(10) @(posedge clk);
	$finish;
end
	
or1200_tmu dut (
	.clk			(clk),
	.rst			(rst),

	.spr_addr		(spr_addr),
	.spr_dat_i		(spr_dat_i),
	.spr_cs			(spr_cs[`OR1200_SPR_GROUP_TMU]),
	.spr_write		(spr_write),
	.spr_dat_o		(spr_dat_o),
	
	.pic_ints(pic_ints),
	.pic_ints_masked(pic_ints_masked),
	
	.intr			(sig_tmu),
	.except_started	(except_started),
	.ex_freeze		(ex_freeze)
);
/*
initial  begin
	$display("\t\ttime,\tclk,\tsig_tmu,\tpic_ints,\tpic_ints_masked"); 
	$monitor("%d,\t%b,\t%b,\t\t%h,\t\t%h",$time, clk, sig_tmu, pic_ints, pic_ints_masked); 
end
*/
 




endmodule
