#include "config.h"

/* System includes */
#include <stdlib.h>
#include <stdint.h>

/* Package includes */
#include "sim-config.h"
#include "execute.h"
#include "sched.h"
#include "except.h"
#include "pic.h"
#include "sprs.h"
#include "tmu.h"

//#define tmutest


#define TMU_SR (cpu_state.sprs[SPR_TMU_STATUS])
#define TMU_CR (cpu_state.sprs[SPR_TMU_CTRL])
#define TMU_CNT	((uint64_t) cpu_state.sprs[SPR_TMU_CNT_H] << 32 | cpu_state.sprs[SPR_TMU_CNT_L])
#define TMU_CMP ((uint64_t) cpu_state.sprs[SPR_TMU_CMP_H] << 32 | cpu_state.sprs[SPR_TMU_CMP_L])
#define PIC_MASK cpu_state.sprs[SPR_TMU_PIC_MASK]
#define TMU_REPCMP 	((uint64_t) cpu_state.sprs[SPR_TMU_REPCMP_H] << 32 | cpu_state.sprs[SPR_TMU_REPCMP_L])
#define TMU_REPCNT ((uint64_t) cpu_state.sprs[SPR_TMU_REPCOUNT_H] << 32 | cpu_state.sprs[SPR_TMU_REPCOUNT_L])

//Internal functions
void tmu_increment_count(int add);
uint32_t tmu_compare();
uint32_t tmu_update_status();
void tmu_raise_except();
void tmu_clear(void);

void tmu_set_repcnt_reg(uint64_t cnt);
void tmu_set_compare_reg(uint64_t cmp);
void tmu_set_count_reg(uint64_t cnt);


void tmu_reset(void)
{
	int i = 0;
	for (i = 0; i < 44; i++)
	{
		cpu_state.sprs[SPRGROUP_TMU+i]= 0;
	}
	cpu_state.sprs[SPR_TMU_STATUS] 	= config.tmu.status;
}

void tmu_main(int add)
{
	tmu_update_status();
	tmu_increment_count(add);
	tmu_compare();
}

void tmu_raise_except()
{
	except_handle(EXCEPT_TMU, cpu_state.sprs[SPR_EEAR_BASE]);
}

void tmu_reassert_interrupt(void * dat)
{
//	printf("TMU::Reaasserting int %d \n", (int*)dat);
	report_interrupt((int*)dat);
}

static int ignored[2] = {0,0};
static int pass[2] = {0,0};
int tmu_int_filter(int line)
{

	if (!(TMU_SR & SPR_TMU_SR_CI))
	{
#ifdef tmutest
		printf("TMU:: Filter:: Count interrupts are disabled :: Int %d\n",line);
#endif

		return 0;
	}
#ifdef tmutest
	printf("TMU:: Filter:: Checking line %d \n \t Budget = %d \t Count = %d ", line, cpu_state.sprs[SPR_TMU_INTRCMP_BASE+line],cpu_state.sprs[SPR_TMU_INTRCNT_BASE+line]);
#endif
	int intMask = 1<<line;
	if (intMask & PIC_MASK)
	{
#ifdef tmutest
		printf("Ignored %d | # %d\n",line, ++ignored[line-3]);
#endif
		return 1;
	}


	cpu_state.sprs[SPR_TMU_INTRCNT_BASE+line]++;
	if (cpu_state.sprs[SPR_TMU_INTRCNT_BASE+line] >= cpu_state.sprs[SPR_TMU_INTRCMP_BASE+line] && cpu_state.sprs[SPR_TMU_INTRCMP_BASE+line] > 0 )
	{
		PIC_MASK |= intMask;
	}
#ifdef tmutest
	printf("Pass:: %d | # %d \n", line, ++pass[line-3]);
#endif
	return 0;
}

void tmu_increment_count(int add)
{
	uint64_t cnt= TMU_CNT;

	uint64_t intCnt = TMU_REPCNT;
	if (config.tmu.use_if && (TMU_REPCMP > 0))
	{
		intCnt+=add;
		tmu_set_repcnt_reg(intCnt);
	}
	if (config.tmu.use_tt && (TMU_SR & SPR_TMU_SR_R))
	{
		cnt+=add;
		tmu_set_count_reg(cnt);
	}
}

uint32_t tmu_compare()
{
	if (config.tmu.use_if && !(TMU_SR & SPR_TMU_SR_CI))
	{
		PIC_MASK = 0;
	}
	int i;
	if (config.tmu.use_if && TMU_REPCNT >= TMU_REPCMP && TMU_REPCMP > 0)
	{
		for ( i = 0; i < 32; i++)
		{
			cpu_state.sprs[SPR_TMU_INTRCNT_BASE+i]= 0;
			if (PIC_MASK & (1<<i))
			{
				PIC_MASK &= ~(1<<i);
				SCHED_ADD(tmu_reassert_interrupt,i, 0);
			}
		}
#ifdef tmutest
		printf("\nTMU:: Replenish -------------\n");
#endif
		tmu_set_repcnt_reg(0);
	}
	if (config.tmu.use_tt && (cpu_state.sprs[SPR_SR] & SPR_SR_CEE) && (TMU_SR & SPR_TMU_SR_EE) &&(TMU_SR & SPR_TMU_SR_R))
	{
		if (TMU_CNT >= TMU_CMP && TMU_CMP > 0)
		{
#ifdef tmutest
			printf("TMU:: Scheduling exception, Compare = %lld\n", TMU_CMP);
#endif
			TMU_SR &= ~SPR_TMU_SR_R;
			SCHED_ADD(tmu_raise_except,NULL,0);
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
}

uint32_t tmu_update_status()
{
	int err = 0;
	//	clear
	if ((TMU_CR & SPR_TMU_CTRL_CLEAR) && !(TMU_SR & SPR_TMU_SR_R))
	{
		tmu_clear();
		TMU_CR &= ~SPR_TMU_CTRL_CLEAR;
	}

	//restart
	if ((TMU_CR & SPR_TMU_CTRL_RESTART) && !(TMU_SR & SPR_TMU_SR_R))
	{
		tmu_set_count_reg(0);
		TMU_CR &= ~SPR_TMU_CTRL_RESTART;
	}

	//stop/start the counter
	if (TMU_CR & SPR_TMU_CTRL_STOP)
	{
		TMU_SR &= ~SPR_TMU_SR_R;
		TMU_CR &= ~SPR_TMU_CTRL_STOP;
	}
	else
	{
		if (TMU_CR & SPR_TMU_CTRL_START)
		{
			TMU_SR |= SPR_TMU_SR_R;
			TMU_CR &= ~SPR_TMU_CTRL_START;
		}
	}

	//Exception enable
	if (TMU_CR & SPR_TMU_CTRL_ED)
	{
		TMU_SR &= ~SPR_TMU_SR_EE;
		TMU_CR &= ~SPR_TMU_CTRL_ED;
	}
	else
	{
		if (TMU_CR & SPR_TMU_CTRL_EE)
		{
			TMU_SR |= SPR_TMU_SR_EE;
			TMU_CR &= ~SPR_TMU_CTRL_EE;
		}
	}

	//freeze
	if (TMU_CR & SPR_TMU_CTRL_FCD)
	{
		TMU_SR &= ~SPR_TMU_SR_FC;
		TMU_CR &= ~SPR_TMU_CTRL_FCD;
	}
	else
	{
		if (TMU_CR & SPR_TMU_CTRL_FCE)
		{
			TMU_SR |= SPR_TMU_SR_FC;
			TMU_CR &= ~SPR_TMU_CTRL_FCE;
		}
	}

	//count exception
	if (TMU_CR & SPR_TMU_CTRL_CED)
	{
		TMU_SR &= ~SPR_TMU_SR_CE;
		TMU_CR &= ~SPR_TMU_CTRL_CED;
	}
	else
	{
		if (TMU_CR & SPR_TMU_CTRL_CEE)
		{
			TMU_SR |= SPR_TMU_SR_CE;
			TMU_CR &= ~SPR_TMU_CTRL_CEE;
		}
	}

	//count interupt

	if (TMU_CR & SPR_TMU_CTRL_CID)
	{
		TMU_SR &= ~SPR_TMU_SR_CI;
		TMU_CR &= ~SPR_TMU_CTRL_CID;
	}
	else
	{
		if (TMU_CR & SPR_TMU_CTRL_CIE)
		{
			TMU_SR |= SPR_TMU_SR_CI;
			TMU_CR &= ~SPR_TMU_CTRL_CIE;
		}
	}

}

void tmu_clear(void)
{
	tmu_set_count_reg(0);
	tmu_set_compare_reg(0);
	tmu_set_repcnt_reg(0);
	TMU_SR &= ~SPR_TMU_SR_CNTI;
}



void tmu_set_compare_reg(uint64_t cmp)
{
	cpu_state.sprs[SPR_TMU_CMP_H] = cmp >> 32;
	cpu_state.sprs[SPR_TMU_CMP_L] = cmp & 0x00000000FFFFFFFF;
}


void tmu_set_count_reg(uint64_t cnt)
{
	cpu_state.sprs[SPR_TMU_CNT_H] = cnt >> 32;
	cpu_state.sprs[SPR_TMU_CNT_L] = cnt & 0x00000000FFFFFFFF;
}

void tmu_set_repcnt_reg(uint64_t cnt)
{
	cpu_state.sprs[SPR_TMU_REPCOUNT_H] = cnt >> 32;
	cpu_state.sprs[SPR_TMU_REPCOUNT_L] = cnt & 0x00000000FFFFFFFF;
}

/*----------------------------------------------------[ TMU configuration ]---*/


/*---------------------------------------------------------------------------*/
/*!Enable or disable the Time Management Unit

   Set the corresponding field in the UPR

   @param[in] val  The value to use
   @param[in] dat  The config data structure (not used here)                 */
/*---------------------------------------------------------------------------*/
static void
tmu_enabled (union param_val  val,
	     void            *dat)
{
  config.tmu.enabled = val.int_val;

}	/* pic_enabled() */

/*---------------------------------------------------------------------------*/
/*!Set the default Status Register value

   @param[in] val  The value to use
   @param[in] dat  The config data structure (not used here)                 */
/*---------------------------------------------------------------------------*/
static void
tmu_set_status (union param_val  val,
		  void            *dat)
{
  config.tmu.status = val.int_val;

}	/* tmu_set_status() */

/*---------------------------------------------------------------------------*/
/*!Enable or disable the task timer

   @param[in] val  The value to use
   @param[in] dat  The config data structure (not used here)                 */
/*---------------------------------------------------------------------------*/
static void
tmu_use_tt (union param_val  val,
		  void            *dat)
{
  config.tmu.use_tt = val.int_val;

}	/* tmu_use_tt() */

/*---------------------------------------------------------------------------*/
/*!Enable or disable the interrupt filter

   @param[in] val  The value to use
   @param[in] dat  The config data structure (not used here)                 */
/*---------------------------------------------------------------------------*/
static void
tmu_use_if(union param_val  val,
		  void            *dat)
{
  config.tmu.use_if = val.int_val;

}	/* tmu_use_if() */

/*  Called by 'info register' in simulator	*/
static void
tmu_status (void *dat){
	printf("TMU:: \nStatus:%lx \nCompare: %llx \nCount: %llx\n", TMU_SR, TMU_CMP, TMU_CNT );
}


/*---------------------------------------------------------------------------*/
/*!Initialize a new interrupt controller configuration

   ALL parameters are set explicitly to default values in init_defconfig()   */
/*---------------------------------------------------------------------------*/


void
reg_tmu_sec ()
{
  struct config_section *sec = reg_config_sec ("tmu", NULL, NULL);

  reg_config_param (sec, "enabled",      PARAMT_INT, tmu_enabled);
  reg_config_param (sec, "status",		 PARAMT_INT, tmu_set_status);
  reg_config_param (sec, "task_timer",   PARAMT_INT, tmu_use_tt);
  reg_config_param (sec, "int_filter",   PARAMT_INT, tmu_use_if);
  reg_sim_stat (tmu_status, NULL);
}	/* reg_pic_sec() */

