
#include "tmu.h"
#include "spr_defs.h"
//#include "support.h"
/*
 * TMU CONTROL FUNCTIONS
 */
void tmu_start()
{
	tmu_set_control(SPR_TMU_CTRL_START);
}

void tmu_stop()
{
	tmu_set_control(SPR_TMU_CTRL_STOP);
}

void tmu_restart()
{

	tmu_set_control(SPR_TMU_CTRL_STOP);
	tmu_set_control(SPR_TMU_CTRL_RESTART | SPR_TMU_CTRL_START);
}

void tmu_clear()
{
	tmu_set_control(SPR_TMU_CTRL_STOP);
	tmu_set_control(SPR_TMU_CTRL_CLEAR);
}

void tmu_enable_exception()
{
	tmu_set_control(SPR_TMU_CTRL_EE);
}

void tmu_disable_exception()
{
	tmu_set_control(SPR_TMU_CTRL_ED);
}

void tmu_enable_count_exception()
{
	tmu_set_control(SPR_TMU_CTRL_CEE);
}
void tmu_disable_count_exception()
{
	tmu_set_control(SPR_TMU_CTRL_CED);
}

void tmu_enable_count_freeze()
{
	tmu_set_control(SPR_TMU_CTRL_FCE);
}

void tmu_disable_count_freeze()
{
	tmu_set_control(SPR_TMU_CTRL_FCD);
}
void tmu_enable_count_interrupt()
{
	tmu_set_control(SPR_TMU_CTRL_CIE);
}
void tmu_disable_count_interrupt()
{
	tmu_set_control(SPR_TMU_CTRL_CID);
}

/*
 * TMU TASK R/W FUNCTIONS
 */
void tmu_set_control(ulong val)
{
	mtspr(SPR_TMU_CTRL, val);
}
ulong tmu_get_status()
{
	return (mfspr(SPR_TMU_STATUS));
}

void tmu_set_compare(ullong val)
{
	tmu_stop();
	ulong low, high;
	low = val & 0x00000000FFFFFFFF;
	high = val >> 32;
	mtspr(SPR_TMU_CMP_H, high);
	mtspr(SPR_TMU_CMP_L, low);
}

ullong tmu_get_compare()
{
	ulong low, high;
	ullong ret;
	low = mfspr(SPR_TMU_CMP_L);
	high= mfspr(SPR_TMU_CMP_H);
	ret = (ullong) high << 32 | low;

	return ret;
}
ullong tmu_get_count()
{
	ulong low, high;
	ullong ret;
	low = mfspr(SPR_TMU_CNT_L);
	high= mfspr(SPR_TMU_CNT_H);
	ret = (ullong) high << 32 | low;

	return ret;
}

/*
 * TMU INTERRUPT R/W FUNCTIONS
 */
void tmu_set_interrupt_rep_period(ullong val)
{
	ulong low, high;
	low = val & 0x00000000FFFFFFFF;
	high = val >> 32;
	mtspr(SPR_TMU_REPCMP_H, high);
	mtspr(SPR_TMU_REPCMP_L, low);
}

ullong tmu_get_interrupt_rep_period()
{
	ulong low, high;
	ullong ret;
	low = mfspr(SPR_TMU_REPCMP_L);
	high= mfspr(SPR_TMU_REPCMP_H);
	ret = (ullong) high << 32 | low;

	return ret;
}
void tmu_set_interrupt_compare(int line, ulong val)
{
	mtspr(SPR_TMU_INTRCMP_BASE+line, val);
}

ulong tmu_get_interrupt_compare(int line)
{
	return (mfspr(SPR_TMU_INTRCMP_BASE+line));
}

ulong tmu_get_interrupt_count(int line)
{
	return (mfspr(SPR_TMU_INTRCNT_BASE+line));
}














