
/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS books - available as PDF or paperback  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

#include <stdlib.h>
#include <string.h>

/* Architecture specific header files. */
#include "support.h"
#include "spr_defs.h"

/* Scheduler header files. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"


/* BSP headers. */
#include "support.h"
#include "board.h"
#include "uart.h"
#include "gpio.h"
#include "dma.h"

#include "interrupts.h"


#define ENDTIME 2000
#define ENDPERCENT ENDTIME/4

//#define dualtask
//#define singletask
//#define maintest
#define tmutest
//#define overhead
//#define freertostest

#ifdef freertostest
#include "freertostest.h"
#endif
#ifdef singletask
#include "singletask.h"
#endif
#ifdef dualtask
#include "dualtask.h"
#endif
#ifdef maintest
#include "maintest.h"
#endif
#ifdef tmutest
#include "tmutest.h"
#endif


/*
 * ST provided routine to configure the processor.
 */
static void prvSetupHardware(void);

void vApplicationIdleHook( void );
void vApplicationTickHook( void );
void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName );
void vApplicationMallocFailedHook( void );


void report_status(char *p, int num);

 int TickCount = 0;
/*-----------------------------------------------------------*/

/* Create all the demo application tasks, then start the scheduler. */
int main( int argc, char **argv )
{
	argc = argc;
	argv = argv;
	/* Perform any hardware setup necessary. */
  	prvSetupHardware();
  	uart_print_str("Hardware setup successfull\n\0");
  	uart_print_str("Starting FreeRTOS\n\0");
#ifdef freertostest
  	ftMain();
#endif
#ifdef tmutest
  	tmutest_main();
#endif
#ifdef dualtask
	dualtask_main();
#endif
#ifdef singletask
	singletask_main();
#endif
#ifdef maintest
	maintest_main();
#endif

	uart_print_str("Starting scheduler\n\0");
	vTaskStartScheduler();
#ifdef maintest
	mtPrint_stats();
#endif
#ifdef dualtask
	dtPrint_stats();
#endif


	or32_exit(0);
	
	/* Execution will only reach here if there was insufficient heap to
	start the scheduler. */
	return 0;
}

void report_status(char *p, int num) {
	uart_print_int(TickCount);
	uart_print_str(": ");
	uart_print_str(p);
	uart_print_int(num);
	uart_print_str("\n\0");
}

/*-----------------------------------------------------------*/

void prvSetupHardware( void )
{
	// UART controller use 50 Mhz Wishbone bus clock, define in board.h
	uart_init(0);			
	uart_rxint_enable(0);

	// Initialize internal Programmable Interrupt Controller
//	int_init();
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
#ifdef freertostest
	ftIdleHook();
#endif
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void ) 
{
	TickCount++;
#ifndef tmutest
	if (TickCount%(ENDPERCENT) == 0) report(0x71c00000|TickCount);
	if (TickCount>=ENDTIME) {
		vTaskEndScheduler();
	}
#endif

}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName ) 
{
	/* prevent compiler warning */
	pxTask = pxTask;
	pcTaskName = pcTaskName;

	report(0x00000099);
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	report(0x00000098);
}
/*-----------------------------------------------------------*/
