


#include <stdlib.h>
#include "FreeRTOS.h"
#include "maintest.h"
#include "interrupts.h"
#include "task.h"
#include "support.h"
#include "intgen.h"
#include "spr_defs.h"

int mtExceptionHelp[3] = {0, 0, 0};

int mtTaskCount[3] = {0};
int mtExceptionCount[3] = {0};
int mtInterruptCount[2]={0};

#ifdef RAND_MAX
#undef RAND_MAX
#endif
#define RAND_MAX  65535
//#define RAND_MAX 268435455
#define mtNOCRITICAL
//#define mtNOINTS
#define mtNORAND

extern void report_status(char*, int);

const unsigned long long replenish = REPLENISHMENT;
xTMUStruct mtTMU1 =
{
	mtException,
	TASK_LIMIT_1,			// Compare
	NULL
};

xTMUStruct mtTMU2 =
{
	mtException,
	TASK_LIMIT_2,	// Compare
	NULL
};

xTMUStruct mtTMU3 =
{
	mtException,
	TASK_LIMIT_3,	// Compare
	NULL
};

void maintest_main()
{
	vTaskEnterCritical();
	report_status("Setting up tasks", 0);
#ifndef mtNORAND
	srand(6189);
#endif
	int_init();
	int one = 1;
	int zero = 0;
	int two = 2;
	int three = 3;
	tmu_disable_count_exception();
#ifndef mtNOINTS
	int_add(INT0, mtIntRoutine, (void *)zero);
	int_add(INT1,  mtIntRoutine, (void *)one);
#endif
	tmu_set_interrupt_rep_period(replenish);
	tmu_set_interrupt_compare(INT0, INT_LIMIT_0);
	tmu_set_interrupt_compare(INT1, INT_LIMIT_1);
	vTaskExitCritical();
#ifndef mtNOINTS
	REG32(INTGEN_BASE_0) = (rand()%RAND_MAX);
	REG32(INTGEN_BASE_1) = (rand()%RAND_MAX);
#endif
	xTaskCreateTMU( mtTaskFunction, "Task1", mtSTACK_DEPTH, (void *)one, mtPRIORITY, NULL, &mtTMU1 );
	xTaskCreateTMU( mtTaskFunction, "Task2", mtSTACK_DEPTH, (void *)two, mtPRIORITY, NULL, &mtTMU2 );
	xTaskCreateTMU( mtTaskFunction, "Task3", mtSTACK_DEPTH, (void *)three, mtPRIORITY, NULL, &mtTMU3 );
}
void mtIntRoutine(void * param)
{


	int intgen = (int *)param;
	mtInterruptCount[intgen]++;
	intgen_clear_interrupt(intgen);
//	intgen_set_timeout (intgen, rand()%RAND_MAX);
//	report(0xa0000000 | intgen << 30);
//	report_status("Interrupt ", intgen);
}



void mtTaskFunction(void * param)
{
	int taskN = (int *) param;
	int fail;
	while (1)
	{
		mtTaskCount[taskN-1]++;
//		report_status("Task ", taskN);
		fail = rand()%100;
#ifndef mtNOCRITICAL
		vPortDisableInterruptsAndTicks();
#endif
		if (fail < 10 ) {
			while (mtExceptionHelp[taskN-1] <=0);
		}
		mtExceptionHelp[taskN-1] = 0;
//		report(0xA115E000 | taskN);
#ifndef mtNOCRITICAL
		vPortEnableInterruptsAndTicks();
#endif
		tmu_restart();
		vTaskDelay( (portTickType) 1);
	}
}


void mtException(void * param)
{
	int taskN = (int *) param;
	mtExceptionHelp[taskN-1] = 1;
	//report_status("TMU exception ", taskN);
	mtExceptionCount[taskN-1]++;
//	report(0xEEEE0000 | taskN);
}

void mtPrint_stats(void)
{
	uart_print_str("Stopping FreeRTOS\n\0");
	uart_print_str("---Stat---\n\0");
	uart_print_str("Ticks: ");
	uart_print_int(TickCount);
	uart_print_str("\n\0");

	uart_print_str("Task 1:\n\0 ");
	uart_print_str("\tRounds: ");
	uart_print_int(mtTaskCount[0]);
	uart_print_str("\n\0");
	uart_print_str("\tExceptions: ");
	uart_print_int(mtExceptionCount[0]);
	uart_print_str("\n\0");

	uart_print_str("Task 2:\n\0 ");
	uart_print_str("\tRounds: ");
	uart_print_int(mtTaskCount[1]);
	uart_print_str("\n\0");
	uart_print_str("\tExceptions: ");
	uart_print_int(mtExceptionCount[1]);
	uart_print_str("\n\0");

	uart_print_str("Task 3:\n\0\ ");
	uart_print_str("\tRounds: ");
	uart_print_int(mtTaskCount[2]);
	uart_print_str("\n\0");
	uart_print_str("\tExceptions: ");
	uart_print_int(mtExceptionCount[2]);
	uart_print_str("\n\0");
	uart_print_str("\n\0");
	uart_print_str("Interrupt 0: ");
	uart_print_int(mtInterruptCount[0]);
	uart_print_str("\n\0");

	uart_print_str("Interrupt 1: ");
	uart_print_int(mtInterruptCount[1]);
	uart_print_str("\n\0");

	uart_print_str("Ignored Interrupt 0: ");
	uart_print_int(mfspr(SPR_PIC_IGNORE_BASE+3));
	uart_print_str("\n\0");

	uart_print_str("Ignored Interrupt 1: ");
	uart_print_int(mfspr(SPR_PIC_IGNORE_BASE+4));
	uart_print_str("\n\0");


}

