/*
 * tmucontroltest.c
 *
 *  Created on: May 15, 2014
 *      Author: kyrre
 */
#include <stdlib.h>
#include "FreeRTOS.h"
#include "task.h"
#include "portable.h"
#include "tmu.h"
#include "semphr.h"

#include "tmucontroltest.h"
#include "spr_defs.h"


#define TMU_SR tmu_get_status()




void tmucontroltest_main()
{
	vSemaphoreCreateBinary(semReturnFromException);
	if( semReturnFromException == NULL )
		{
			uart_print_str("Main:: Failed to initiate semaphore \n ");
		}
	xTaskCreateException(&tmucontroltestParam, NULL);
	xSemaphoreTake(semReturnFromException,(portTickType)1);
}

xTMUStruct tmucontroltestTMU =
{
		tmucontroltest_exeption,
	0,	 		// Count High
	0, 			// Count Low
	0, 			// Compare High
	0,	// Compare Low
	0x00000000 	// Control
//	0101 0101 0000	0001

};

const xTaskParameters tmucontroltestParam =
{
	(tmucontroltest_task),		// pvTaskCode - the function that implements the task.
	("HighPri"),	// pcName - just a text name for the task to assist debugging.
	(configMINIMAL_STACK_SIZE),		// usStackDepth	- the stack size DEFINED IN WORDS.
	(NULL),		// pvParameters - passed into the task function as the function parameters.
	(DELAY_PRIORITY+1),// uxPriority - task priority, set the portPRIVILEGE_BIT if the task should run in a privileged state.
	(NULL),// puxStackBuffer - the buffer to be used as the task stack.

	// xRegions - Allocate up to three separate memory regions for access by
	// the task, with appropriate access permissions.  Different processors have
	// different memory alignment requirements - refer to the FreeRTOS documentation
	// for full information.
	{NULL},
	&tmucontroltestTMU
};

void busywait()
{
	int i = 0;
	for (i = 0; i < 10; i++)
	{
		i++;
		i--;
	}
}

void tmucontroltest_task(void * param)
{

	int err = 0;
	udlong cmp, cnt;
	/*
	 * Default state tests
	 *
	 */
	if (tmu_get_count() != 0) err++;
	if (tmu_get_compare() != 0) err++;
	if (tmu_get_status() != 0x16) err++;
/*
 *Control and status behavior checks
 */

	tmu_set_control(SPR_TMU_CTRL_STOP | SPR_TMU_CTRL_ED | SPR_TMU_CTRL_CED | SPR_TMU_CTRL_CID | SPR_TMU_CTRL_FCD);
	if (tmu_get_status() != 0) err++;

	tmu_stop();
	if ((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R ) err++;

	tmu_enable_exception();
	if (!((TMU_SR & SPR_TMU_SR_EE) == SPR_TMU_SR_EE )) err++;

	tmu_disable_exception();
	if ((TMU_SR & SPR_TMU_SR_EE) == SPR_TMU_SR_EE ) err++;

	tmu_enable_count_exception();
	if (!((TMU_SR & SPR_TMU_SR_CE) == SPR_TMU_SR_CE )) err++;

	tmu_disable_count_exception();
	if ((TMU_SR & SPR_TMU_SR_CE) == SPR_TMU_SR_CE) err++;

	tmu_enable_count_freeze();
	if (!((TMU_SR & SPR_TMU_SR_FC) == SPR_TMU_SR_FC )) err++;

	tmu_disable_count_freeze();
	if ((TMU_SR & SPR_TMU_SR_FC) == SPR_TMU_SR_FC ) err++;

	tmu_enable_count_interrupt();
	if (!((TMU_SR & SPR_TMU_SR_CI) == SPR_TMU_SR_CI )) err++;

	tmu_disable_count_interrupt();
	if ((TMU_SR & SPR_TMU_SR_CI) == SPR_TMU_SR_CI ) err++;

	if (tmu_get_status() != 0 )err++;

	//tmu_reset();
	//if (tmu_get_count() != 0) err++;
	//if (tmu_get_compare() != 0) err++;
	//if (tmu_get_status() != 0x16) err++;

/*
 * Compare and count R/W checks
 */
	//tmu_reset();
	//if (tmu_get_count() != 0) err++;
	//if (tmu_get_compare() != 0) err++;
	//if (tmu_get_status() != 0x16) err++;

	//Set compare,
	cmp=10;
	tmu_set_compare(cmp);
	if (tmu_get_compare()!= cmp) err++;

	cmp=0;
	tmu_set_compare(cmp);
	if (tmu_get_compare()!= cmp) err++;

	cmp=0x1001100110011001;
	tmu_set_compare(cmp);
	if (tmu_get_compare()!= cmp) err++;

	tmu_clear();
	if (tmu_get_compare() !=0 | tmu_get_count() != 0) err++;


/*
 * Simple task operation test
 */
	//tmu_reset();
	//if (tmu_get_count() != 0) err++;
	//if (tmu_get_compare() != 0) err++;
	//if (tmu_get_status() != 0x16) err++;

	//Start counting
	cmp = 1337;
	tmu_set_compare(cmp);
	if (tmu_get_compare()!= cmp) err++;
	tmu_start();
	if (!((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R )) err++;
	busywait();
	if (!(tmu_get_count() > 0)) err++;

	//Stop and see if cnt holds over time
	tmu_stop();
	if ((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R ) err++;
	cnt = tmu_get_count();
	busywait();
	if (tmu_get_count() != cnt) err++;

	//Start and stop test
	tmu_start();
	if (!((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R )) err++;
	tmu_stop();
	if ((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R ) err++;

	//Restarting
	tmu_stop();
	if ((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R ) err++;
	tmu_restart();
	if ((tmu_get_count() > 0)) err++;
	tmu_start();
	if (!((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R )) err++;
	busywait();
	if (!(tmu_get_count() > 0)) err++;

	//Clear during runtime
	tmu_stop();
	if ((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R ) err++;
	tmu_clear();
	if (tmu_get_compare() !=0 | tmu_get_count() != 0) err++;


	//No Exception test
	cmp = 1000;
	tmu_set_compare(cmp);
	if (tmu_get_compare()!= cmp) err++;
	tmu_start();
	if (!((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R )) err++;

	while (tmu_get_count() < cmp)
	{

	}
	tmu_stop();
	if ((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R ) err++;
	tmu_clear();
	if (tmu_get_compare() !=0 | tmu_get_count() != 0) err++;

	//Generate exception
	tmu_enable_exception();
	cmp = 1000;
	tmu_set_compare(cmp);
	if (tmu_get_compare()!= cmp) err++;
	tmu_start();
	if (!((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R )) err++;

	//return from exception here


	while(!(xSemaphoreTake(semReturnFromException,(portTickType)1) == pdTRUE));

	tmu_stop();
	if ((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R ) err++;
	tmu_clear();
	if (tmu_get_compare() !=0 | tmu_get_count() != 0) err++;




	uart_print_str("Test completed with ");
	uart_print_int(err);
	uart_print_str(" errors \n");
	vTaskEndScheduler();
}


void tmucontroltest_exeption(){
	report(0xFFFFFFFF);
	if ((TMU_SR & SPR_TMU_SR_R) == SPR_TMU_SR_R ) report(0xE5505000);
	xSemaphoreGive(semReturnFromException);
}
