#include <stdlib.h>
#include "FreeRTOS.h"
#include "semphr.h"
#include "uart.h"
#include "support.h"
#include "tmu.h"
#include "spr_defs.h"

#include "tmutest.h"

typedef unsigned long long ullong;
typedef unsigned long ulong;

#define TICK_PERIOD configCPU_CLOCK_HZ/configTICK_RATE_HZ
#define INT3_BASE 0xa0000000
#define INT3 3

#define INT4 4
#define INT4_BASE 0xa1000000
const ullong replenish = 10*TICK_PERIOD;

int Exception;
int Interrupt[2]= {0};
int tmu_driver_test();
int tt_behavior_test();
int if_behavior_test();
void ttIntRoutine(void * param);

void tmutest_main()
{
	int_init();
	int three = 3, four = 4;
	int_add(INT3, ttIntRoutine,(void*)three);
	int_add(INT4, ttIntRoutine,(void*)four);
  	xTaskCreate( tmutest_func, "tmutest", configMINIMAL_STACK_SIZE , NULL, ttPRIORITY, NULL );
}

void tmutest_func(void * param)
{
	int driverError = 0, ttError = 0, ifError = 0;
	Exception = 0;
	param=param;
	driverError = tmu_driver_test();
	uart_print_str("tmu_driver_test returns with ");
	uart_print_int(driverError);
	uart_print_str(" errors\n\0 ");
	ttError = tt_behavior_test();
	uart_print_str("tt_behavior_test returns with ");
	uart_print_int(ttError);
	uart_print_str(" errors\n\0 ");
	ifError = if_behavior_test();
	uart_print_str("if_behavior_test returns with ");
	uart_print_int(ifError);
	uart_print_str(" errors\n\0 ");


	uart_print_str("TMU test finished with ");
	uart_print_int(driverError+ttError+ifError);
	uart_print_str(" errors \n");
	vTaskEndScheduler();
}

void ttIntRoutine(void * param)
{
	int taskN = (int*)param;
	Interrupt[taskN-3]=1;

}

int if_behavior_test()
{
	int error = 0;
	tmu_enable_count_interrupt();
	if (!(tmu_get_status() & SPR_TMU_SR_CI)) error++;

	tmu_set_interrupt_rep_period(replenish);

	tmu_set_interrupt_compare(INT3, 2);
	if (tmu_get_interrupt_count(INT3) != 0) error++;
	if (tmu_get_interrupt_compare(INT3) != 2) error++;

	tmu_set_interrupt_compare(INT4, 3);
	if (tmu_get_interrupt_count(INT4) != 0) error++;
	if (tmu_get_interrupt_compare(INT4) != 3) error++;
	int next_tick= 0;
	int interruptCount = 0;
	REG32(INT3_BASE) = 37500;
	int enabled=1;
	int state = 0;
	while (1)
	{
		switch (state)
		{
		case 0:
			if (Interrupt[0] == 1)
			{
				REG32(INT3_BASE) = 37500;
				interruptCount++;
				Interrupt[0]=0;
			}
			if (interruptCount == 4)
			{
				interruptCount = 0;
				state = 1;
				next_tick=TickCount+15;
			}
			break;
		case 1:
			if (Interrupt[0] == 1)
			{
				interruptCount++;
				Interrupt[0]=0;
				REG32(INT3_BASE) = 37500;
			}
			if (TickCount >= next_tick)
			{
				next_tick= TickCount+1;

			}

			if (interruptCount == 5 && !enabled)
			{
				enabled = 1;
				tmu_enable_count_interrupt();
			}
			if (interruptCount == 1 && enabled)
			{
				enabled = 0;
				tmu_disable_count_interrupt();
			}
			if (interruptCount == 6 )
			{
				interruptCount = 0;
				state = 2;
				next_tick=TickCount+15;
				tmu_enable_count_interrupt();
				break;
			}
			break;
		case 2:
			if (Interrupt[0] == 1)
			{
				interruptCount++;
				Interrupt[0]=0;
			}
			if (Interrupt[1] == 1)
			{
				interruptCount++;
				Interrupt[1]=0;
			}

			if (TickCount >= next_tick)
			{
				next_tick= TickCount+1;
				REG32(INT4_BASE) = 30000;
				REG32(INT3_BASE) = 37500;
			}
			if (interruptCount >= 6)
			{
				return error;
			}
			break;
		}
	}
}



int tt_behavior_test()
{
	int error = 0;
	int state = 2, next_state = 1, prev_state = 0;
	int exceptionCount=0;
	int startTick = 0;
	mtspr(SPR_SR, (mfspr(SPR_SR) | SPR_SR_CEE)); /* Enable TMU interrupts*/
	error += mfsprCheck(SPR_SR, (mfspr(SPR_SR) | SPR_SR_CEE));
	tmu_enable_exception();
	tmu_enable_count_exception();
	tmu_clear();
	while(1){
		switch(state)
		{
		case 0:
			if (Exception==1)
			{
				tmu_clear();
				state=next_state;
				exceptionCount++;
				Exception = 0;
				if (prev_state == 4 || prev_state == 5 || prev_state == 6) error++;
			}
			if (prev_state== 4 || prev_state == 5 )
			{
				if (TickCount == startTick+1)
				{
					tmu_clear();
					state=next_state;
					tmu_enable_exception();
					mtspr(SPR_SR, (mfspr(SPR_SR) | SPR_SR_CEE));
				}
			}
			if (prev_state == 6)
			{
				if (TickCount == startTick+1)
				{
					state= next_state;
				}
			}
			break;
		case 1:
			tmu_stop();
			if (exceptionCount != 3) error++;

			return error;
		case 2:
			prev_state = 2;
			state=0;
			next_state=3;
			tmu_set_compare(400000);
			tmu_start();
			break;
		case 3:
			state=0;
			next_state=4;
			prev_state = 3;
			tmu_set_compare(0x100000001);//0x10003);//
			tmu_start();
			break;
		case 4:
			state=0;
			prev_state = 4;
			next_state=5;
			startTick=TickCount;
			tmu_disable_exception();
			tmu_set_compare(0x10004);
			tmu_start();
			break;
		case 5:
			state=0;
			prev_state = 5;
			next_state=6;
			startTick=TickCount;
			mtspr(SPR_SR, (mfspr(SPR_SR) & ~SPR_SR_CEE));
			tmu_set_compare(0x10005);
			tmu_start();
			break;
		case 6:
			state = 0;
			prev_state = 6;
			next_state = 7;
			startTick=TickCount;
			tmu_set_compare(600000);
			tmu_start();
			break;
		case 7:
			state = 0;
			prev_state= 7;
			next_state= 1;
			tmu_stop();
			tmu_set_compare(1200000);
			tmu_start();
			break;
		}
	}
}

void tmutest_exception()
{
	tmu_stop();
	Exception= 1;
}

int	mfsprCheck(ulong reg, ulong val)
{
	if (!(mfspr(reg) == val)) return 1;
	return 0;

}

void forloop(){
	int i = 0;
	for (i = 0; i < 10; i++)
	{
		;;
	}
}

int tmu_driver_test()
{
	int error = 0;
	ulong ttStatus = 0x12; 	/*	Task Timer Status 	*/
	ulong ttControl = 0;	/* 	Task Timer Control	*/
	ullong ttCount = 0; 	/*	Task Timer Count 	*/
	ullong ttCompare = 0;	/* 	Task Timer Compare 	*/
	ullong ifRepCompare = 0; /* Interrupt filter replenishment compare */

	/*
	 * TMU CONTROL FUNCTIONS
	 */
	/* Check initial values*/
	ttStatus = mfspr(SPR_TMU_STATUS);
	if (ttStatus != (SPR_TMU_SR_EE | SPR_TMU_SR_CI)) error++;
	error += mfsprCheck(SPR_TMU_STATUS, (SPR_TMU_SR_EE | SPR_TMU_SR_CI)); /* Verify the assist function */

	error += mfsprCheck(SPR_TMU_CMP_L, 0);
	error += mfsprCheck(SPR_TMU_CMP_H, 0);
	error += mfsprCheck(SPR_TMU_CNT_L, 0);
	error += mfsprCheck(SPR_TMU_CNT_H, 0);
	error += mfsprCheck(SPR_TMU_CTRL, 0);

	/* Check enable disable functions */
	tmu_set_control(SPR_TMU_CTRL_START);
	error += mfsprCheck(SPR_TMU_CTRL, 0);
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus | SPR_TMU_SR_R);

	tmu_set_control(SPR_TMU_CTRL_STOP);
	error += mfsprCheck(SPR_TMU_CTRL, 0);
	error += mfsprCheck(SPR_TMU_STATUS, (ttStatus & ~SPR_TMU_SR_R));

	tmu_start();
	ttStatus |= SPR_TMU_SR_R;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);
	ttCount = mfspr(SPR_TMU_CNT_L);
	if (mfspr(SPR_TMU_CNT_L) <= 0) error++;

	tmu_start();
	ttStatus |= SPR_TMU_SR_R;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);
	if (mfspr(SPR_TMU_CNT_L) <= ttCount) error++;

	tmu_stop();
	ttStatus &= ~SPR_TMU_SR_R;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);
	ttCount=mfspr(SPR_TMU_CNT_L);
	forloop();
	if (ttCount != mfspr(SPR_TMU_CNT_L)) error++;

	tmu_stop();
	ttStatus &= ~SPR_TMU_SR_R;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);
	ttCount=mfspr(SPR_TMU_CNT_L);
	forloop();
	if (ttCount != mfspr(SPR_TMU_CNT_L)) error++;


	tmu_disable_exception();
	ttStatus &= ~SPR_TMU_SR_EE;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	tmu_enable_exception();
	ttStatus |= SPR_TMU_SR_EE;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	tmu_enable_count_exception();
	ttStatus |= SPR_TMU_SR_CE;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	tmu_disable_count_exception();
	ttStatus &= ~SPR_TMU_SR_CE;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	tmu_enable_count_freeze();
	ttStatus |= SPR_TMU_SR_FC;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	tmu_disable_count_freeze();
	ttStatus &= ~SPR_TMU_SR_FC;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	tmu_enable_count_interrupt();
	ttStatus |= SPR_TMU_SR_CI;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	tmu_disable_count_interrupt();
	ttStatus &= ~SPR_TMU_SR_CI;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	/* Get / set functions*/
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);
	if (!(tmu_get_status()== ttStatus)) error++;

	/* Read/Write Compare high and low*/
	error += mfsprCheck(SPR_TMU_CMP_L, 0);
	error += mfsprCheck(SPR_TMU_CMP_H, 0);
	if (!(tmu_get_compare() == 0)) error++;


	ttCompare = 0xFFFFdcaf141adca1;
	tmu_set_compare(ttCompare);
	error += mfsprCheck(SPR_TMU_CMP_L,(ttCompare & 0x0FFFFFFFF));
	error += mfsprCheck(SPR_TMU_CMP_H,(ttCompare>>32));
	if (!(tmu_get_compare() == ttCompare)) error++;
	ttStatus |= SPR_TMU_SR_CNTI;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);

	/* Setting count */
	tmu_stop();
	ttCount = 0x000be1f4a51ab141;
	mtspr(SPR_TMU_CNT_L, (ttCount & 0x0FFFFFFFF));
	mtspr(SPR_TMU_CNT_H, (ttCount>>32));
	ttStatus &= ~SPR_TMU_SR_CNTI;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);


	ttCount= tmu_get_count();
	error += mfsprCheck(SPR_TMU_CNT_L,(ttCount & 0x0FFFFFFFF));
	error += mfsprCheck(SPR_TMU_CNT_H,(ttCount>>32));

	/* Reading and restarting count  */
	tmu_start();
	if(tmu_get_count() == 0) error++;
	forloop();
	ttCount=tmu_get_count();
	tmu_restart();
	ttStatus |= SPR_TMU_SR_R;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);
	if (ttCount <= tmu_get_count()) error++;
	if (tmu_get_count()== 0) error++;

	/* Clear */
	tmu_clear();
	ttStatus &= ~SPR_TMU_SR_R;
	error += mfsprCheck(SPR_TMU_STATUS, ttStatus);
	if (tmu_get_compare()!= 0 || tmu_get_count() != 0 ) error++;
	forloop();
	if (tmu_get_compare()!= 0 || tmu_get_count() != 0 ) error++;

	/* Writing count and compare while running*/
	ttCount= 0x14578178911498;
	tmu_start();
	mtspr(SPR_TMU_CNT_L, (ttCount & 0x0FFFFFFFF));
	mtspr(SPR_TMU_CNT_H, (ttCount>>32));
	if(tmu_get_count() == ttCount) error++;
	ttCompare = 0x1490175123;
	mtspr(SPR_TMU_CMP_L, (ttCompare & 0x0FFFFFFFF));
	mtspr(SPR_TMU_CMP_H, (ttCompare>>32));
	if(tmu_get_compare() == ttCompare) error++;

	/* Checking initial value interrupt filter*/
	error += mfsprCheck(SPR_TMU_REPCMP_L,0);
	error += mfsprCheck(SPR_TMU_REPCMP_H,0);
	error += mfsprCheck(SPR_TMU_REPCOUNT_L,0);
	error += mfsprCheck(SPR_TMU_REPCOUNT_H,0);
	error += mfsprCheck(SPR_TMU_PIC_MASK,0);
	int line = 0;
	for (line = 0; line < 32; line++)
	{
		error += mfsprCheck(SPR_TMU_INTRCMP_BASE+line,0);
		error += mfsprCheck(SPR_TMU_INTRCNT_BASE+line,0);
	}

	if (tmu_get_interrupt_rep_period() != 0 ) error++;

	/* Checking setting of compare/count values */

	ifRepCompare = 0xab5d0ffbcde5fe56;
	tmu_set_interrupt_rep_period(ifRepCompare);
	if (!(tmu_get_interrupt_rep_period() == ifRepCompare)) error++;

	ulong val = 0xe528fbae;
	tmu_set_interrupt_rep_period(0); /* Stop rep count*/
	tmu_clear();
	mtspr(SPR_TMU_REPCOUNT_H, val);
	mtspr(SPR_TMU_REPCOUNT_L, val);
	error += mfsprCheck(SPR_TMU_REPCOUNT_L,0);
	error += mfsprCheck(SPR_TMU_REPCOUNT_H,0);



	for (line = 0; line < 32; line++)
	{
		tmu_set_interrupt_compare(line, val);
		error += mfsprCheck(SPR_TMU_INTRCMP_BASE+line,val);
		if (!(tmu_get_interrupt_compare(line) == val)) error++;


		mtspr(SPR_TMU_INTRCNT_BASE+line, val);
		error += mfsprCheck(SPR_TMU_INTRCNT_BASE+line,0);
		if (!(tmu_get_interrupt_count(line)==0)) error++;
	}
	return error;
}



